@@ -42,7 +42,7 @@ It handles initialization and termination by subclassing wxApp.
 #include <unistd.h>
 #endif
 
-// chmod
+// chmod, lstat, geteuid
 #ifdef __UNIX__
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -72,6 +72,8 @@ It handles initialization and termination by subclassing wxApp.
 #include "FileNames.h"
 #include "AutoRecovery.h"
 #include "SplashDialog.h"
+#include "FFT.h"
+#include "BlockFile.h"
 
 #ifdef EXPERIMENTAL_MODULES
 #include "LoadModules.h"
@@ -82,6 +84,14 @@ It handles initialization and termination by subclassing wxApp.
 #include "import/ImportQT.h"
 #endif
 
+#ifdef _DEBUG
+    #ifdef _MSC_VER
+        #undef THIS_FILE
+        static char*THIS_FILE= __FILE__;
+        #define new new(_NORMAL_BLOCK, THIS_FILE, __LINE__)
+    #endif
+#endif
+
 // Windows specific linker control...only needed once so
 // this is a good place (unless we want to add another file).
 #if defined(__WXMSW__)
@@ -142,13 +152,13 @@ It handles initialization and termination by subclassing wxApp.
 #  endif
 
 #  if defined(USE_LIBTWOLAME)
-#     pragma comment(lib, "libtwolame_static" U D)
+#     pragma comment(lib, "twolame_static" U D)
 #  endif
 
 #  undef U
 #  undef D
 
-#endif 
+#endif //(__WXMSW__)
 
 #if wxUSE_ACCESSIBILITY
 #if wxCHECK_VERSION(2, 8, 0)
@@ -341,7 +351,7 @@ public:
    };
 };
 
-#endif
+#endif //__WXMSW__
 
 IMPLEMENT_APP(AudacityApp)
 /* make the application class known to wxWidgets for dynamic construction */
@@ -414,7 +424,7 @@ void AudacityApp::OnMacOpenFile(wxCommandEvent & event)
       MRUOpen(name);
    }
 }
-#endif
+#endif //__WXMAC__
 
 typedef int (AudacityApp::*SPECIALKEYEVENT)(wxKeyEvent&);
 
@@ -531,6 +541,33 @@ void AudacityApp::OnMRUProject(wxCommandEvent& event) {
 #endif
 
 
+void AudacityApp::InitLang( const wxString & lang )
+{
+   if( mLocale )
+      delete mLocale;
+
+   if (lang != wxT("en")) {
+      wxLogNull nolog;
+      mLocale = new wxLocale(wxT(""), lang, wxT(""), true, true);
+
+      for(unsigned int i=0; i<audacityPathList.GetCount(); i++)
+         mLocale->AddCatalogLookupPathPrefix(audacityPathList[i]);
+
+#ifdef AUDACITY_NAME
+      mLocale->AddCatalog(wxT(AUDACITY_NAME));
+#else
+      mLocale->AddCatalog(wxT("audacity"));
+#endif
+   } else
+      mLocale = NULL;
+
+   // Initialize internationalisation (number formats etc.)
+   //
+   // This must go _after_ creating the wxLocale instance because
+   // creating the wxLocale instance sets the application-wide locale.
+   Internat::Init();
+}
+
 // The `main program' equivalent, creating the windows and returning the
 // main frame
 bool AudacityApp::OnInit()
@@ -566,15 +603,14 @@ bool AudacityApp::OnInit()
    // AColor depends on theTheme.
    AColor::Init(); 
 
-   // On Unix systems, the default temp dir is in /tmp.
-   // Search path (in this order):
-   // * The AUDACITY_PATH environment variable
-   // * The current directory
-   // * The user's .audacity-files directory in their home directory
-   // * The "share" and "share/doc" directories in their install path
+   /* On Unix systems, the default temp dir is in /tmp. */
+   /* Search path (for plug-ins, translations etc) is (in this order):
+      * The AUDACITY_PATH environment variable
+      * The current directory
+      * The user's .audacity-files directory in their home directory
+      * The "share" and "share/doc" directories in their install path */
    #ifdef __WXGTK__
-   defaultTempDir.Printf(wxT("/tmp/audacity%d.%d-%s"), 
-                         AUDACITY_VERSION, AUDACITY_RELEASE, wxGetUserId().c_str());
+   defaultTempDir.Printf(wxT("/tmp/audacity-%s"), wxGetUserId().c_str());
    
    wxString pathVar = wxGetenv(wxT("AUDACITY_PATH"));
    if (pathVar != wxT(""))
@@ -590,20 +626,20 @@ bool AudacityApp::OnInit()
       AddUniquePathToPathList(wxString::Format(wxT("%s/share/doc/%s"),
                                                wxT(INSTALL_PREFIX), wxT(AUDACITY_NAME)),
                               audacityPathList);
-   #else
+   #else //AUDACITY_NAME
       AddUniquePathToPathList(wxString::Format(wxT("%s/share/audacity"),
                                                wxT(INSTALL_PREFIX)),
                               audacityPathList);
       AddUniquePathToPathList(wxString::Format(wxT("%s/share/doc/audacity"),
                                                wxT(INSTALL_PREFIX)),
                               audacityPathList);
-   #endif
+   #endif //AUDACITY_NAME
 
    AddUniquePathToPathList(wxString::Format(wxT("%s/share/locale"),
                                             wxT(INSTALL_PREFIX)),
                            audacityPathList);
 
-   #endif
+   #endif //__WXGTK__
 
    wxFileName tmpFile;
    tmpFile.AssignTempFileName(wxT("nn"));
@@ -617,9 +653,9 @@ bool AudacityApp::OnInit()
    AddUniquePathToPathList(progPath, audacityPathList);
    AddUniquePathToPathList(progPath+wxT("\\Languages"), audacityPathList);
    
-   defaultTempDir.Printf(wxT("%s\\audacity_%d_%d_temp"), 
-                         tmpDirLoc.c_str(), AUDACITY_VERSION, AUDACITY_RELEASE);
-   #endif
+   defaultTempDir.Printf(wxT("%s\\audacity_temp"), 
+                         tmpDirLoc.c_str());
+   #endif //__WXWSW__
    #ifdef __MACOSX__
    // On Mac OS X, the path to the Audacity program is in argv[0]
    wxString progPath = wxPathOnly(argv[0]);
@@ -631,20 +667,19 @@ bool AudacityApp::OnInit()
 
    AddUniquePathToPathList(progPath+wxT("/Languages"), audacityPathList);
    AddUniquePathToPathList(progPath+wxT("/../../../Languages"), audacityPathList);
-   defaultTempDir.Printf(wxT("%s/audacity%d.%d-%s"), 
+   defaultTempDir.Printf(wxT("%s/audacity-%s"), 
                          tmpDirLoc.c_str(),
-                         AUDACITY_VERSION, AUDACITY_RELEASE, 
                          wxGetUserId().c_str());
-   #endif
+   #endif //__MACOSX__
    #ifdef __MACOS9__
    // On Mac OS 9, the initial working directory is the one that
    // contains the program.
    wxString progPath = wxGetCwd();
    AddUniquePathToPathList(progPath, audacityPathList);
    AddUniquePathToPathList(progPath+wxT(":Languages"), audacityPathList);
-   defaultTempDir.Printf(wxT("%s/audacity_%d_%d_temp"), 
-                         tmpDirLoc.c_str(), AUDACITY_VERSION, AUDACITY_RELEASE, );
-   #endif
+   defaultTempDir.Printf(wxT("%s/audacity_temp"), 
+                         tmpDirLoc.c_str());
+   #endif //__MACOS9__
 
    // BG: Create a temporary window to set as the top window
    wxFrame *temporarywindow = new wxFrame(NULL, -1, wxT("temporarytopwindow"));
@@ -680,26 +715,8 @@ bool AudacityApp::OnInit()
 #endif
    gPrefs->Write(wxT("/Locale/Language"), lang);
 
-   if (lang != wxT("en")) {
-      wxLogNull nolog;
-      mLocale = new wxLocale(wxT(""), lang, wxT(""), true, true);
-
-      for(unsigned int i=0; i<audacityPathList.GetCount(); i++)
-         mLocale->AddCatalogLookupPathPrefix(audacityPathList[i]);
-
-#ifdef AUDACITY_NAME
-      mLocale->AddCatalog(wxT(AUDACITY_NAME));
-#else
-      mLocale->AddCatalog(wxT("audacity"));
-#endif
-   } else
-      mLocale = NULL;
-
-   // Initialize internationalisation (number formats etc.)
-   //
-   // This must go _after_ creating the wxLocale instance because
-   // creating the wxLocale instance sets the application-wide locale.
-   Internat::Init();
+   mLocale = NULL;
+   InitLang( lang );
 
    // Init DirManager, which initializes the temp directory
    // If this fails, we must exit the program.
@@ -741,7 +758,7 @@ bool AudacityApp::OnInit()
    gParentFrame->Hide();
 
    SetTopWindow(gParentFrame);
-#endif
+#endif //__WXMAC__
 
    SetExitOnFrameDelete(true);
 
@@ -757,7 +774,7 @@ bool AudacityApp::OnInit()
       pWnd->Show( true );
    }
    else
-#endif   
+#endif  //EXPERIMENTAL_MODULES 
    {
       SetTopWindow(project);
       project->Show( true );
@@ -853,7 +870,7 @@ bool AudacityApp::OnInit()
       }                         // for option...
    }                            // if (argc>1)
 
-#else
+#else //__CYGWIN__
 	
    // Cygwin command line parser (by Dave Fancella)
    if (argc > 1 && !didRecoverAnything) {
@@ -930,7 +947,7 @@ bool AudacityApp::OnInit()
       }                         // for option...
    }                            // if (argc>1)
 
-#endif // Cygwin command-line parser
+#endif // __CYGWIN__ (Cygwin command-line parser)
 
    gInited = true;
    
@@ -946,7 +963,7 @@ bool AudacityApp::InitCleanSpeech()
    #ifdef __WXGTK__
    if (presetsFromPrefs.GetChar(0) != wxT('/'))
       presetsFromPrefs = wxT("");
-   #endif
+   #endif //__WXGTK__
 
    wxString presetsDefaultLoc =
       wxFileName(userdatadir, wxT("presets")).GetFullPath();
@@ -1009,7 +1026,7 @@ bool AudacityApp::InitTempDir()
    if (tempFromPrefs != wxT("")) {
       if (wxDirExists(tempFromPrefs))
          temp = tempFromPrefs;
-      else if (wxMkdir(tempFromPrefs))
+      else if (wxMkdir(tempFromPrefs, 0755))
          temp = tempFromPrefs;
    }
 
@@ -1018,10 +1035,23 @@ bool AudacityApp::InitTempDir()
    if (temp==wxT("") && tempDefaultLoc != wxT("")) {
       if (wxDirExists(tempDefaultLoc))
          temp = tempDefaultLoc;
-      else if (wxMkdir(tempDefaultLoc))
+      else if (wxMkdir(tempDefaultLoc, 0755))
          temp = tempDefaultLoc;
    }
 
+   // Check temp directory ownership on *nix systems only
+   #ifdef __UNIX__
+   struct stat tempStatBuf;
+   if ( lstat(temp.mb_str(), &tempStatBuf) != 0 ) {
+      temp.clear();
+   }
+   else {
+      if ( geteuid() != tempStatBuf.st_uid ) {
+         temp.clear();
+      }
+   }
+   #endif
+
    if (temp == wxT("")) {
       // Failed
       wxMessageBox(_("Audacity could not find a place to store temporary files.\nPlease enter an appropriate directory in the preferences dialog."));
@@ -1289,9 +1319,14 @@ int AudacityApp::OnExit()
 
    UnloadEffects();
 
+   DeinitFFT();
+   BlockFile::Deinit();
+
    DeinitAudioIO();
    Internat::CleanUp();// JKC
 
+   if (mLocale)
+      delete mLocale;
    delete mChecker;
 
    return 0;
