@@ -407,7 +407,7 @@ bool Envelope::HandleMouseButtonDown(wxMouseEvent & event, wxRect & r,
          else
             mContourOffset = false;
       }
-      
+
       double newVal = ValueOfPixel(clip_y, r.height, upper, dB,
                                    zoomMin, zoomMax, eMin);
       
@@ -534,7 +534,7 @@ bool Envelope::MouseEvent(wxMouseEvent & event, wxRect & r,
                           float zoomMin, float zoomMax, float eMin, float eMax)
 {
 
-   if (event.ButtonDown() && mButton == wxMOUSE_BTN_NONE)
+   if (event.ButtonDown())
       return HandleMouseButtonDown( event, r, h, pps,dB,
                                     zoomMin, zoomMax, eMin, eMax);
 
@@ -597,6 +597,8 @@ void Envelope::Paste(double t0, Envelope *e)
       // msmeyer: The envelope is empty and has the same default value, so
       // there is nothing that must be inserted, just return. This avoids
       // the creation of unnecessary duplicate control points
+      // MJS: but the envelope does get longer
+      mTrackLen += e->mTrackLen;
       return;
    }
    if (this->mEnv.Count() != 0)
@@ -841,6 +843,10 @@ void Envelope::GetPoints(double *bufferWhen,
 // If you call it from Envelope, don't. Kind of makes sense. MJS.
 int Envelope::Insert(double when, double value)
 {
+   // in debug builds, do a spot of argument checking
+   wxASSERT(when <= (mTrackLen));
+   wxASSERT(when >= 0);
+
    int len = mEnv.Count();
 
    if (len && when < 0.0)
@@ -928,6 +934,10 @@ void Envelope::GetValues(double *buffer, int bufferLen,
 
    double tprev, vprev, tnext = 0, vnext, vstep = 0;
 
+   // in debug builds, do a spot of argument checking
+   wxASSERT(t0 <= (mTrackLen));
+   wxASSERT(t0 >= 0);
+
    for (int b = 0; b < bufferLen; b++) {
       if (len <= 0) {
          buffer[b] = mDefaultValue;
