@@ -54,6 +54,7 @@ for drawing different aspects of the label and its text box.
 #include "Theme.h"
 #include "AllThemeResources.h"
 #include "AColor.h"
+#include "Project.h"
 
 wxFont LabelTrack::msFont;
 
@@ -1282,7 +1283,20 @@ bool LabelTrack::CaptureKey(wxKeyEvent & event)
          break;
       }
 
-      if (IsGoodLabelCharacter(keyCode, charCode) && !event.CmdDown()) {
+      if( IsGoodLabelCharacter(keyCode, charCode) && !event.CmdDown()) {
+         return true;
+      }
+   }
+   else
+   {
+      if( IsGoodLabelFirstCharacter(keyCode, charCode) && !event.CmdDown() ){
+         AudacityProject * pProj = GetActiveProject();
+         // IF Label already there, then don't add a new one on typing.
+         if( GetLabelIndex( pProj->mViewInfo.sel0,  pProj->mViewInfo.sel1) != wxNOT_FOUND )
+            return false;
+
+         if( pProj )
+            pProj->OnAddLabel();
          return true;
       }
    }
@@ -2001,6 +2015,31 @@ const LabelStruct *LabelTrack::GetLabel(int index) const
    return mLabels[index];
 }
 
+int LabelTrack::GetLabelIndex(double t, double t1)
+{
+   LabelStruct *l;
+
+   int len = mLabels.Count();
+   int i;
+   //We'd have liked to have times in terms of samples, 
+   //because then we're doing an intrger comparison.
+   //Never mind.  Instead we look for near enough.
+   //This level of (in)accuracy is only a problem if we 
+   //deal with sounds in the MHz range.
+   const double delta = 1.0e-7; 
+   for( i=0;i<len;i++)
+   {
+      l = mLabels[i];
+      if( fabs( l->t - t ) > delta )
+         continue;
+      if( fabs( l->t1 - t1 ) > delta )
+         continue;
+      return i;
+   }
+
+   return wxNOT_FOUND;
+}
+
 int LabelTrack::AddLabel(double t, double t1, const wxString &title)
 {
    LabelStruct *l = new LabelStruct();
