@@ -116,6 +116,7 @@ void GetLanguages(wxArrayString &langCodes, wxArrayString &langNames)
    localLanguageName[wxT("ja")] = wxT("Nihongo");
    localLanguageName[wxT("lo")] = wxT("Lao"); // aka Laothian
    localLanguageName[wxT("lt")] = wxT("Lietuviu");
+   localLanguageName[wxT("he")] = wxT("Hebrew");
    localLanguageName[wxT("hu")] = wxT("Magyar");
    localLanguageName[wxT("mk")] = wxT("Makedonski");
    localLanguageName[wxT("nl")] = wxT("Nederlands");
@@ -129,6 +130,7 @@ void GetLanguages(wxArrayString &langCodes, wxArrayString &langNames)
    localLanguageName[wxT("sv")] = wxT("Svenska");
    localLanguageName[wxT("tr")] = wxT("Turkce");
    localLanguageName[wxT("uk")] = wxT("Ukrainska");
+   localLanguageName[wxT("vi")] = wxT("Vietnamese");
    localLanguageName[wxT("zh")] = wxT("Chinese (Simplified)");
    localLanguageName[wxT("zh_TW")] = wxT("Chinese (Traditional)");
 
@@ -189,6 +191,28 @@ void GetLanguages(wxArrayString &langCodes, wxArrayString &langNames)
       }
    }
 
+
+   // JKC: Adding language for simplified audacity.
+   {
+      wxString code;
+      wxString name;
+      code = wxT("en-simple");
+      name = wxT("Simplified");
+      if (TranslationExists(audacityPathList, code) ) {
+         tempCodes.Add(code);
+         tempNames.Add(name);
+         tempHash[code] = name;
+
+         /* for debugging
+         printf(wxT("code=%s name=%s fullCode=%s name=%s -> %s\n"),
+                code.c_str(), localLanguageName[code].c_str(),
+                fullCode.c_str(), localLanguageName[fullCode].c_str(),
+                name.c_str());
+         */
+      }
+   }
+
+
    // Sort
 
    unsigned int j;
