@@ -64,6 +64,14 @@
 #include <wx/textfile.h>
 #include <wx/combobox.h>
 
+#ifdef _DEBUG
+    #ifdef _MSC_VER
+        #undef THIS_FILE
+        static char*THIS_FILE= __FILE__;
+        #define new new(_NORMAL_BLOCK, THIS_FILE, __LINE__)
+    #endif
+#endif
+
 static const wxChar *DefaultGenres[] =
 {
    wxT("Blues"),
@@ -326,7 +334,7 @@ int Tags::GetNumGenres()
 void Tags::LoadDefaultGenres()
 {
    mGenres.Clear();
-   for (int i = 0; i < WXSIZEOF(DefaultGenres); i++) {
+   for (size_t i = 0; i < WXSIZEOF(DefaultGenres); i++) {
       mGenres.Add(DefaultGenres[i]);
    }
 }
@@ -542,7 +550,7 @@ void Tags::WriteXML(XMLWriter &xmlFile)
 
 bool Tags::ShowEditDialog(wxWindow *parent, wxString title, bool force)
 {
-   if (force || IsEmpty()) {
+   if (force) {
       TagsEditor dlg(parent, title, this, mEditTitle, mEditTrackNumber);
 
       return dlg.ShowModal() == wxID_OK;
@@ -626,7 +634,7 @@ public:
 #define LABEL_GENRE     _("Genre")
 #define LABEL_COMMENTS  _("Comments")
 
-static const wxChar *names[] =
+static wxString names[] =
 {
    LABEL_ARTIST,
    LABEL_TITLE,
@@ -637,10 +645,10 @@ static const wxChar *names[] =
    LABEL_COMMENTS
 };
 
-static const struct
+static struct
 {
-   const wxChar *label;
-   const wxChar *name;
+   wxString label;
+   wxString name;
 }
 labelmap[] =
 {
@@ -652,6 +660,7 @@ labelmap[] =
    {  LABEL_GENRE,      TAG_GENRE      },
    {  LABEL_COMMENTS,   TAG_COMMENTS   }
 };
+
 #define STATICCNT WXSIZEOF(labelmap)
 
 enum {
@@ -690,6 +699,30 @@ TagsEditor::TagsEditor(wxWindow * parent,
    mEditTitle(editTitle),
    mEditTrack(editTrack)
 {
+   names[0] = LABEL_ARTIST;
+   names[1] = LABEL_TITLE;
+   names[2] = LABEL_ALBUM;
+   names[3] = LABEL_TRACK;
+   names[4] = LABEL_YEAR;
+   names[5] = LABEL_GENRE;
+   names[6] = LABEL_COMMENTS;
+
+   labelmap[0].label = LABEL_ARTIST;
+   labelmap[1].label = LABEL_TITLE;
+   labelmap[2].label = LABEL_ALBUM;
+   labelmap[3].label = LABEL_TRACK;
+   labelmap[4].label = LABEL_YEAR;
+   labelmap[5].label = LABEL_GENRE;
+   labelmap[6].label = LABEL_COMMENTS;
+
+   labelmap[0].name = TAG_ARTIST;
+   labelmap[1].name = TAG_TITLE;
+   labelmap[2].name = TAG_ALBUM;
+   labelmap[3].name = TAG_TRACK;
+   labelmap[4].name = TAG_YEAR;
+   labelmap[5].name = TAG_GENRE;
+   labelmap[6].name = TAG_COMMENTS;
+
    mGrid = NULL;
 
    // Make a local copy of the passed in tags
@@ -866,7 +899,7 @@ bool TagsEditor::TransferDataFromWindow()
 
 bool TagsEditor::TransferDataToWindow()
 {
-   int i;
+   size_t i;
    wxString n;
    wxString v;
 
@@ -934,7 +967,7 @@ void TagsEditor::OnChange(wxGridEvent & event)
    }
 
    wxString n = mGrid->GetCellValue(event.GetRow(), 0);
-   for (int i = 0; i < STATICCNT; i++) {
+   for (size_t i = 0; i < STATICCNT; i++) {
       if (n.CmpNoCase(labelmap[i].label) == 0) {
          ischanging = true;
          wxBell();
@@ -1011,7 +1044,6 @@ void TagsEditor::OnReset(wxCommandEvent & event)
    wxFileName fn(FileNames::DataDir(), wxT("genres.txt"));
    wxTextFile tf(fn.GetFullPath());
 
-   bool saved = false;
    bool open = (tf.Exists() && tf.Open()) ||
                (!tf.Exists() && tf.Create());
 
@@ -1208,7 +1240,7 @@ void TagsEditor::OnAdd(wxCommandEvent & event)
 
 void TagsEditor::OnRemove(wxCommandEvent & event)
 {
-   int row = mGrid->GetCursorRow();
+   size_t row = mGrid->GetCursorRow();
 
    if (!mEditTitle && mGrid->GetCellValue(row, 0).CmpNoCase(LABEL_TITLE) == 0) {
       return;
@@ -1655,7 +1687,6 @@ void TagsEditor1::OnReset(wxCommandEvent & event)
    wxFileName fn(FileNames::DataDir(), wxT("genres.txt"));
    wxTextFile tf(fn.GetFullPath());
 
-   bool saved = false;
    bool open = (tf.Exists() && tf.Open()) ||
                (!tf.Exists() && tf.Create());
 
@@ -2389,7 +2420,6 @@ void TagsEditor2::OnReset(wxCommandEvent & event)
    wxFileName fn(FileNames::DataDir(), wxT("genres.txt"));
    wxTextFile tf(fn.GetFullPath());
 
-   bool saved = false;
    bool open = (tf.Exists() && tf.Open()) ||
                (!tf.Exists() && tf.Create());
 
