@@ -57,11 +57,11 @@
 
 #undef PROFILE_WAVEFORM
 #ifdef PROFILE_WAVEFORM
-	#ifdef __WXMSW__
-		#include <time.h>
-	#else
-		#include <sys/time.h>
-	#endif
+   #ifdef __WXMSW__
+      #include <time.h>
+   #else
+      #include <sys/time.h>
+   #endif
 double gWaveformTimeTotal = 0;
 int gWaveformTimeCount = 0;
 #endif
@@ -84,8 +84,20 @@ TrackArtist::TrackArtist()
    mInsetRight  = 0;
    mInsetBottom = 0;
 
+   mdBrange = ENV_DB_RANGE;
+   mShowClipping = false;
+   UpdatePrefs();
+
    SetColours();
    vruler = new Ruler();
+
+#ifdef EXPERIMENTAL_FFT_Y_GRID
+   fftYGridOld=true;
+#endif //EXPERIMENTAL_FFT_Y_GRID
+
+#ifdef EXPERIMENTAL_FIND_NOTES
+   fftFindNotesOld=false;
+#endif
 }
 
 TrackArtist::~TrackArtist()
@@ -111,6 +123,8 @@ void TrackArtist::SetColours()
    theTheme.SetPenColour(   rmsPen,          clrRms);
    theTheme.SetPenColour(   muteRmsPen,      clrMuteRms);
    theTheme.SetPenColour(   shadowPen,       clrShadow);
+   theTheme.SetPenColour(   clippedPen,      clrClipped);
+   theTheme.SetPenColour(   muteClippedPen,  clrMuteClipped);
 }
 
 void TrackArtist::SetInset(int left, int top, int right, int bottom)
@@ -236,10 +250,23 @@ void TrackArtist::DrawTracks(TrackList * tracks,
                             drawEnvelope,  drawSamples, drawSliders, true, muted);
                break;
             case WaveTrack::SpectrumDisplay:
+#ifdef LOGARITHMIC_SPECTRUM
+               DrawSpectrum((WaveTrack *)t, dc, rr, viewInfo, false, false);
+#else
                DrawSpectrum((WaveTrack *)t, dc, rr, viewInfo, false);
+#endif
                break;
+#ifdef LOGARITHMIC_SPECTRUM
+            case WaveTrack::SpectrumLogDisplay:
+               DrawSpectrum((WaveTrack *)t, dc, rr, viewInfo, false, true);
+               break;
+#endif
             case WaveTrack::PitchDisplay:
+#ifdef LOGARITHMIC_SPECTRUM
+               DrawSpectrum((WaveTrack *)t, dc, rr, viewInfo, true, false);
+#else
                DrawSpectrum((WaveTrack *)t, dc, rr, viewInfo, true);
+#endif
                break;
             }
             break;              // case Wave
@@ -284,6 +311,9 @@ void TrackArtist::DrawVRuler(Track *t, wxDC * dc, wxRect & r)
       vruler->SetFormat(Ruler::RealFormat);
       vruler->SetUnits(wxT(""));
       vruler->SetLabelEdges(false);
+#ifdef LOGARITHMIC_SPECTRUM
+      vruler->SetLog(false);
+#endif
       vruler->Draw(*dc);
    }
 
@@ -292,7 +322,6 @@ void TrackArtist::DrawVRuler(Track *t, wxDC * dc, wxRect & r)
       // Waveform (db)
       vruler->SetUnits(wxT(""));
 
-      float dBr = gPrefs->Read(wxT("/GUI/EnvdBRange"), ENV_DB_RANGE);
       float min, max;
       ((WaveTrack *)t)->GetDisplayBounds(&min, &max);
 
@@ -300,7 +329,7 @@ void TrackArtist::DrawVRuler(Track *t, wxDC * dc, wxRect & r)
          int top = 0;
          float topval = 0;
          int bot = r.height;
-         float botval = -dBr;
+         float botval = -mdBrange;
 
          if (min < 0) {
             bot = top + (int)((max / (max-min))*(bot-top));
@@ -313,18 +342,21 @@ void TrackArtist::DrawVRuler(Track *t, wxDC * dc, wxRect & r)
          }
 
          if (max < 1)
-            topval = -((1-max)*dBr);
+            topval = -((1-max)*mdBrange);
 
          if (min > 0) {
-            botval = -((1-min)*dBr);
+            botval = -((1-min)*mdBrange);
          }
 
          if (topval > botval && bot > top+10) {
             vruler->SetBounds(r.x, r.y+top+1, r.x + r.width, r.y + bot-1);
             vruler->SetOrientation(wxVERTICAL);
             vruler->SetRange(topval, botval);
-            vruler->SetFormat(Ruler::RealFormat);
+            vruler->SetFormat(Ruler::LinearDBFormat);
             vruler->SetLabelEdges(true);
+#ifdef LOGARITHMIC_SPECTRUM
+            vruler->SetLog(false);
+#endif
             vruler->Draw(*dc);
          }
       }
@@ -336,12 +368,24 @@ void TrackArtist::DrawVRuler(Track *t, wxDC * dc, wxRect & r)
       if (r.height < 60)
          return;
 
+#ifdef LOGARITHMIC_SPECTRUM
+      bool logF = ((WaveTrack *) t)->GetDisplay() == WaveTrack::SpectrumLogDisplay;
+#endif
       double rate = ((WaveTrack *) t)->GetRate();
+#ifdef EXPERIMENTAL_FFT_SKIP_POINTS
+      int fftSkipPoints = gPrefs->Read(wxT("/Spectrum/FFTSkipPoints"), 0L);
+#endif //EXPERIMENTAL_FFT_SKIP_POINTS
       int freq = lrint(rate/2.);
       int maxFreq = gPrefs->Read(wxT("/Spectrum/MaxFreq"), freq);
+#ifdef EXPERIMENTAL_FFT_SKIP_POINTS
+      maxFreq/=(fftSkipPoints+1);
+#endif //EXPERIMENTAL_FFT_SKIP_POINTS
       if(maxFreq > freq)
          maxFreq = freq;
       int minFreq = gPrefs->Read(wxT("/Spectrum/MinFreq"), 0L);
+#ifdef EXPERIMENTAL_FFT_SKIP_POINTS
+      minFreq/=(fftSkipPoints+1);
+#endif //EXPERIMENTAL_FFT_SKIP_POINTS
       if(minFreq < 0) {
          minFreq = 0;
          gPrefs->Write(wxT("/Spectrum/MinFreq"), 0L);
@@ -365,10 +409,53 @@ void TrackArtist::DrawVRuler(Track *t, wxDC * dc, wxRect & r)
          vruler->SetRange(int(maxFreq), int(minFreq));
          vruler->SetUnits(wxT(""));
       }
-
+#ifdef LOGARITHMIC_SPECTRUM
+      vruler->SetLog(false);
+#endif
       vruler->Draw(*dc);
    }
 
+#ifdef LOGARITHMIC_SPECTRUM
+   if (t->GetKind() == Track::Wave
+   && ((WaveTrack *) t)->GetDisplay() == WaveTrack::SpectrumLogDisplay)
+   {  // SpectrumLog
+      if (r.height < 10)
+         return;
+      double rate = ((WaveTrack *) t)->GetRate();
+#ifdef EXPERIMENTAL_FFT_SKIP_POINTS
+      int fftSkipPoints = gPrefs->Read(wxT("/Spectrum/FFTSkipPoints"), 0L);
+#endif //EXPERIMENTAL_FFT_SKIP_POINTS
+      int freq = lrint(rate/2.);
+      int maxFreq = gPrefs->Read(wxT("/SpectrumLog/MaxFreq"), freq);
+#ifdef EXPERIMENTAL_FFT_SKIP_POINTS
+      maxFreq/=(fftSkipPoints+1);
+#endif //EXPERIMENTAL_FFT_SKIP_POINTS
+      if(maxFreq > freq)
+         maxFreq = freq;
+      int minFreq = gPrefs->Read(wxT("/SpectrumLog/MinFreq"), freq/1000.0);
+#ifdef EXPERIMENTAL_FFT_SKIP_POINTS
+      minFreq/=(fftSkipPoints+1);
+#endif //EXPERIMENTAL_FFT_SKIP_POINTS
+      if(minFreq < 1) {
+         minFreq = 1;
+           gPrefs->Write(wxT("/SpectrumLog/MinFreq"), 1L);
+      }
+      /*
+         draw the ruler
+         we will use Hz if maxFreq is < 2000, otherwise we represent kHz,
+         and append to the numbers a "k"
+      */
+      vruler->SetBounds(r.x, r.y+1, r.x + r.width, r.y + r.height-1);
+      vruler->SetOrientation(wxVERTICAL);
+      vruler->SetFormat(Ruler::IntFormat);
+      vruler->SetLabelEdges(true);
+      vruler->SetRange(maxFreq, minFreq);
+      vruler->SetUnits(wxT(""));
+      vruler->SetLog(true);
+      vruler->Draw(*dc);
+   }
+#endif
+
    if (t->GetKind() == Track::Wave
        && ((WaveTrack *) t)->GetDisplay() == WaveTrack::PitchDisplay) {
       // Pitch
@@ -452,6 +539,9 @@ void TrackArtist::DrawVRuler(Track *t, wxDC * dc, wxRect & r)
          }
       }
    }
+#ifdef EXPERIMENTAL_RULER_AUTOSIZE
+   t->vrulerSize = vruler->mRect.GetSize();
+#endif //EXPERIMENTAL_RULER_AUTOSIZE
 }
 
 // Takes a value between -1      and     +1 and returns a value between 
@@ -487,8 +577,8 @@ int TrackArtist::GetWaveYPos(float value, int height, bool dB, float dBr)
 /// then use the zoomMin, zoomMax and height values without having 
 /// to have them passed in to it??
 int GetWaveYPosNew(float value, float min, float max,
-		   int height, bool dB, bool outer,
-		   float dBr, bool clip)
+         int height, bool dB, bool outer,
+         float dBr, bool clip)
 {
    float sign = (value >= 0 ? 1 : -1);
 
@@ -523,7 +613,7 @@ int GetWaveYPosNew(float value, float min, float max,
 // This function isn't used anymore.  Use GetWaveYPosNew instead.
 // (and pass in clip=false)
 int TrackArtist::GetWaveYPosUnclipped(float value, int height, bool dB, 
-				      float dBr)
+                  float dBr)
 {
    float sign = (value >= 0 ? 1 : -1);
 
@@ -589,7 +679,6 @@ void TrackArtist::DrawWaveformBackground(wxDC &dc, wxRect r,
    int *mintop = new int[r.width];
    int *minbot = new int[r.width];
    int x;
-   float dBr = gPrefs->Read(wxT("/GUI/EnvdBRange"), ENV_DB_RANGE);
 
    // First we compute the truncated shape of the waveform background.
    // If drawEnvelope is true, then we compute the lower border of the
@@ -597,14 +686,14 @@ void TrackArtist::DrawWaveformBackground(wxDC &dc, wxRect r,
 
    for (x = 0; x < r.width; x++) {
       maxtop[x] = GetWaveYPosNew(env[x], zoomMin, zoomMax,
-                                 r.height, dB, true, dBr, true);
+                                 r.height, dB, true, mdBrange, true);
       maxbot[x] = GetWaveYPosNew(env[x], zoomMin, zoomMax,
-                                 r.height, dB, false, dBr, true);
+                                 r.height, dB, false, mdBrange, true);
 
       mintop[x] = GetWaveYPosNew(-env[x]-.000000001, zoomMin, zoomMax,
-                                 r.height, dB, false, dBr, true);
+                                 r.height, dB, false, mdBrange, true);
       minbot[x] = GetWaveYPosNew(-env[x], zoomMin, zoomMax,
-                                 r.height, dB, true, dBr, true);
+                                 r.height, dB, true, mdBrange, true);
 
       if (!drawEnvelope || maxbot[x] > mintop[x]) {
          maxbot[x] = halfHeight;
@@ -729,7 +818,6 @@ void TrackArtist::DrawIndividualClipSamples(wxDC &dc, wxRect r,
    sampleCount s0 = (sampleCount) (t0 * rate + 0.5);
    sampleCount slen = (sampleCount) (r.width * rate / pps + 0.5);
    sampleCount snSamples = clip->GetNumSamples(); 
-   float dBr = gPrefs->Read(wxT("/GUI/EnvdBRange"), ENV_DB_RANGE);
    
    slen += 4;
 
@@ -743,8 +831,14 @@ void TrackArtist::DrawIndividualClipSamples(wxDC &dc, wxRect r,
 
    int *xpos = new int[slen];
    int *ypos = new int[slen];
+   int *clipped;
+   int clipcnt = 0;
    sampleCount s;
-   
+
+   if (mShowClipping) {
+       clipped = new int[slen];
+   }
+
    dc.SetPen(muted ? muteSamplePen : samplePen);
    
    for (s = 0; s < slen; s++) {
@@ -761,8 +855,12 @@ void TrackArtist::DrawIndividualClipSamples(wxDC &dc, wxRect r,
       
       xpos[s] = xx;
 
-      ypos[s] = GetWaveYPosNew(buffer[s] * clip->GetEnvelope()->GetValueAtX(xx+r.x, r, h, pps),
-                               zoomMin, zoomMax, r.height, dB, true, dBr, false);
+      tt = buffer[s] * clip->GetEnvelope()->GetValueAtX(xx + r.x, r, h, pps);
+      if (mShowClipping && (tt <= -MAX_AUDIO || tt >= MAX_AUDIO)) {
+         clipped[clipcnt++] = xx;
+      }
+      ypos[s] = GetWaveYPosNew(tt, zoomMin, zoomMax,
+                               r.height, dB, true, mdBrange, false);
       if (ypos[s] < -1)
          ypos[s] = -1;
       if (ypos[s] > r.height)
@@ -775,7 +873,7 @@ void TrackArtist::DrawIndividualClipSamples(wxDC &dc, wxRect r,
       dc.DrawLine(r.x + xpos[s], r.y + ypos[s],
                   r.x + xpos[s + 1], r.y + ypos[s + 1]);
    }
-   
+
    if (showPoints) {
       // Draw points
       int tickSize= drawSamples ? 4 : 3;// Bigger ellipses when draggable.
@@ -792,7 +890,20 @@ void TrackArtist::DrawIndividualClipSamples(wxDC &dc, wxRect r,
          }
       }
    }
+
+   // Draw clipping
+   if (clipcnt) {
+      dc.SetPen(muted ? muteClippedPen : clippedPen);
+      while (--clipcnt >= 0) {
+         s = clipped[clipcnt];
+         dc.DrawLine(r.x + s, r.y, r.x + s, r.y + r.height);
+      }
+   }
    
+   if (mShowClipping) {
+      delete [] clipped;
+   }
+
    delete[]buffer;
    delete[]xpos;
    delete[]ypos;
@@ -810,18 +921,39 @@ void TrackArtist::DrawMinMaxRMS(wxDC &dc, wxRect r, uchar *imageBuffer,
    int *h2 = new int[r.width];
    int *r1 = new int[r.width];
    int *r2 = new int[r.width];
+   int *clipped;
+   int clipcnt = 0;
    int x;
-   float dBr = gPrefs->Read(wxT("/GUI/EnvdBRange"), ENV_DB_RANGE);
+
+   if (mShowClipping) {
+       clipped =  new int[r.width];
+   }
 
    for (x = 0; x < r.width; x++) {
-      h1[x] = GetWaveYPosNew(min[x] * envValues[x], zoomMin, zoomMax,
-                             r.height, dB, true, dBr, true);
-      h2[x] = GetWaveYPosNew(max[x] * envValues[x], zoomMin, zoomMax,
-                             r.height, dB, true, dBr, true);
+      double v;
+
+      v = min[x] * envValues[x];
+      if (mShowClipping && v <= -MAX_AUDIO) {
+         if (clipcnt == 0 || (clipcnt > 0 && clipped[clipcnt-1] != x)) {
+            clipped[clipcnt++] = x;
+         }
+      }
+      h1[x] = GetWaveYPosNew(v, zoomMin, zoomMax,
+                             r.height, dB, true, mdBrange, true);
+
+      v = max[x] * envValues[x];
+      if (mShowClipping && v >= MAX_AUDIO) {
+         if (clipcnt == 0 || (clipcnt > 0 && clipped[clipcnt-1] != x)) {
+            clipped[clipcnt++] = x;
+         }
+      }
+      h2[x] = GetWaveYPosNew(v, zoomMin, zoomMax,
+                             r.height, dB, true, mdBrange, true);
+
       r1[x] = GetWaveYPosNew(-rms[x] * envValues[x], zoomMin, zoomMax,
-                             r.height, dB, true, dBr, true);
+                             r.height, dB, true, mdBrange, true);
       r2[x] = GetWaveYPosNew(rms[x] * envValues[x], zoomMin, zoomMax,
-                             r.height, dB, true, dBr, true);
+                             r.height, dB, true, mdBrange, true);
 
       // JKC: This adjustment to h1[] and h2[] ensures that the drawn
       // waveform is continuous.
@@ -843,11 +975,13 @@ void TrackArtist::DrawMinMaxRMS(wxDC &dc, wxRect r, uchar *imageBuffer,
    }
 
    if (imageBuffer) {
+      uchar *clipBuffer = imageBuffer;
       uchar rs, gs, bs, rr, gr, br;
       int y;
 
       GetColour(muted ? muteSamplePen : samplePen, &rs, &gs, &bs);
       GetColour(muted ? muteRmsPen : rmsPen, &rr, &gr, &br);
+
       for(y=0; y<r.height; y++) {
          for(x=0; x<r.width; x++) {
             if (y >= r2[x] && y < r1[x]) {
@@ -858,10 +992,29 @@ void TrackArtist::DrawMinMaxRMS(wxDC &dc, wxRect r, uchar *imageBuffer,
             else if (y >= h2[x] && y < h1[x]+1) {
                *imageBuffer++ = rs;
                *imageBuffer++ = gs;
-               *imageBuffer++ = bs;               
+               *imageBuffer++ = bs;
             }
-            else
+            else {
                imageBuffer += 3;
+            }
+         }
+      }
+
+      if (clipcnt) {
+         uchar rc, gc, bc;
+         int c;
+         int w = r.width * 3;
+
+         GetColour(muted ? muteClippedPen : clippedPen, &rc, &gc, &bc);
+
+         for (c = 0; c < clipcnt; c++) {
+            x = clipped[c] * 3;
+            for (y = 0; y < r.height; y++) {
+               clipBuffer[x] = rc;
+               clipBuffer[x + 1] = gc;
+               clipBuffer[x + 2] = bc;
+               x += w;
+            }
          }
       }
    }
@@ -888,8 +1041,21 @@ void TrackArtist::DrawMinMaxRMS(wxDC &dc, wxRect r, uchar *imageBuffer,
          if (r1[x] != r2[x])
             dc.DrawLine(r.x + x, r.y + r2[x], r.x + x, r.y + r1[x]);
       }
+
+      // Draw the clipping lines
+      if (clipcnt) {
+         dc.SetPen(muted ? muteClippedPen : clippedPen);
+         while (--clipcnt >= 0) {
+            x = clipped[clipcnt];
+            dc.DrawLine(r.x + x, r.y, r.x + x, r.y + r.height);
+         }
+      }
    }
       
+   if (mShowClipping) {
+      delete[] clipped;
+   }
+
    delete[] h1;
    delete[] h2;
    delete[] r1;
@@ -1164,15 +1330,15 @@ void TrackArtist::DrawClipWaveform(WaveTrack* track, WaveClip* clip,
    if (drawEnvelope) {
       dc.SetPen(AColor::envelopePen);
       int x;
-      float dBr = gPrefs->Read(wxT("/GUI/EnvdBRange"), ENV_DB_RANGE);
+
       for (x = 0; x < mid.width; x++) {
          int envTop, envBottom;
          
          envTop = GetWaveYPosNew(envValues[x], zoomMin, zoomMax,
-                                 mid.height, dB, true, dBr, false);
+                                 mid.height, dB, true, mdBrange, false);
          
          envBottom = GetWaveYPosNew(-envValues[x], zoomMin, zoomMax,
-                                    mid.height, dB, true, dBr, false);
+                                    mid.height, dB, true, mdBrange, false);
          
          /* make the collision at zero actually look solid */
          if(envBottom-envTop < 3){
@@ -1186,36 +1352,6 @@ void TrackArtist::DrawClipWaveform(WaveTrack* track, WaveClip* clip,
       }
    }
 
-   // Draw clipping indicators if requested
-   bool showclipping = gPrefs->Read(wxT("/GUI/ShowClipping"), 0L)!=0;
-   if (showclipping) {
-      dc.SetPen(AColor::clippingPen);
-
-      sampleCount s0 = (sampleCount) lrint(t0 * rate);
-      sampleCount slen = (sampleCount) lrint(mid.width * rate / pps);
-      sampleCount snSamples = clip->GetNumSamples(); 
-
-      if (s0 + slen > snSamples) {
-         slen = snSamples - s0;
-      }
-
-      float *buffer = new float[slen];
-      clip->GetSamples((samplePtr)buffer, floatSample, s0, slen);
-
-      for (sampleCount s = 0; s < slen; s++) {
-         float v = floor(buffer[s]);
-         if (buffer[s] <= -MAX_AUDIO) {
-            int x = lrint((s / rate + s0 / rate - t0) * pps);
-            dc.DrawLine(mid.x + x, mid.y + (mid.height / 2), mid.x + x, mid.y + mid.height);
-         }
-         else if (buffer[s] >= MAX_AUDIO) {
-            int x = lrint((s / rate + s0 / rate - t0) * pps);
-            dc.DrawLine(mid.x + x, mid.y, mid.x + x, mid.y + (mid.height / 2));
-         }
-      }
-      delete [] buffer;
-   }
-
    // Draw arrows on the left side if the track extends to the left of the
    // beginning of time.  :)
    if (h == 0.0 && tOffset < 0.0) {
@@ -1315,7 +1451,11 @@ void TrackArtist::DrawTimeSlider(WaveTrack *track,
 
 void TrackArtist::DrawSpectrum(WaveTrack *track,
                                wxDC & dc, wxRect & r,
+#ifdef LOGARITHMIC_SPECTRUM
+                               ViewInfo * viewInfo, bool autocorrelation, bool logF)
+#else
                                ViewInfo * viewInfo, bool autocorrelation)
+#endif
 {
    // MM: Draw background. We should optimize that a bit more.
    dc.SetPen(*wxTRANSPARENT_PEN);
@@ -1336,12 +1476,40 @@ void TrackArtist::DrawSpectrum(WaveTrack *track,
    }
 
    for (WaveClipList::Node* it=track->GetClipIterator(); it; it=it->GetNext())
+#ifdef LOGARITHMIC_SPECTRUM
+      DrawClipSpectrum(track, it->GetData(), dc, r, viewInfo, autocorrelation, logF);
+#else
       DrawClipSpectrum(track, it->GetData(), dc, r, viewInfo, autocorrelation);
+#endif
+}
+
+float sumFreqValues(float *freq, int x0, float bin0, float bin1)
+{
+   float value;
+   if (int(bin1) == int(bin0)) {
+      value = freq[x0+int(bin0)];
+   } else {
+      float binwidth= bin1 - bin0;
+      value = freq[x0 + int(bin0)] * (1.f - bin0 + (int)bin0);
+
+      bin0 = 1 + int (bin0);
+      while (bin0 < int(bin1)) {
+         value += freq[x0 + int(bin0)];
+         bin0 += 1.0;
+      }
+      value += freq[x0 + int(bin1)] * (bin1 - int(bin1));
+      value /= binwidth;
+   }
+   return value;
 }
 
 void TrackArtist::DrawClipSpectrum(WaveTrack* track, WaveClip *clip,
                                wxDC & dc, wxRect & r,
+#ifdef LOGARITHMIC_SPECTRUM
+                               ViewInfo * viewInfo, bool autocorrelation, bool logF)
+#else
                                ViewInfo * viewInfo, bool autocorrelation)
+#endif
 {
    double h = viewInfo->h;
    double pps = viewInfo->zoom;
@@ -1448,6 +1616,12 @@ void TrackArtist::DrawClipSpectrum(WaveTrack* track, WaveClip *clip,
    unsigned char *data = image->GetData();
 
    int windowSize = gPrefs->Read(wxT("/Spectrum/FFTSize"), 256);
+#ifdef EXPERIMENTAL_FFT_SKIP_POINTS
+   int fftSkipPoints = gPrefs->Read(wxT("/Spectrum/FFTSkipPoints"), 0L);
+#endif //EXPERIMENTAL_FFT_SKIP_POINTS
+#ifdef EXPERIMENTAL_FFT_Y_GRID
+   bool fftYGrid = (gPrefs->Read(wxT("/Spectrum/FFTYGrid"), 0L) != 0);
+#endif //EXPERIMENTAL_FFT_Y_GRID
    int half = windowSize/2;
    float *freq = new float[mid.width * half];
    sampleCount *where = new sampleCount[mid.width+1];
@@ -1457,6 +1631,30 @@ void TrackArtist::DrawClipSpectrum(WaveTrack* track, WaveClip *clip,
    bool isGrayscale = false;
    gPrefs->Read(wxT("/Spectrum/Grayscale"), &isGrayscale, false);
    int ifreq = lrint(rate/2);
+#ifdef LOGARITHMIC_SPECTRUM
+   int maxFreq;
+   if (!logF)
+      maxFreq = gPrefs->Read(wxT("/Spectrum/MaxFreq"), ifreq);
+   else
+      maxFreq = gPrefs->Read(wxT("/SpectrumLog/MaxFreq"), ifreq);
+   if(maxFreq > ifreq)
+      maxFreq = ifreq;
+   int minFreq;
+   if (!logF) {
+      minFreq = gPrefs->Read(wxT("/Spectrum/MinFreq"), 0L);
+      if(minFreq < 0) {
+         minFreq = 0;
+         gPrefs->Write(wxT("/Spectrum/MinFreq"), 0L);
+      }
+   }
+   else {
+      minFreq = gPrefs->Read(wxT("/SpectrumLog/MinFreq"), ifreq/1000.0);
+      if(minFreq < 1) {
+         minFreq = ifreq/1000.0;
+         gPrefs->Write(wxT("/SpectrumLog/MinFreq"), ifreq/1000.0);
+      }
+   }
+#else
    int maxFreq = gPrefs->Read(wxT("/Spectrum/MaxFreq"), ifreq);
    if(maxFreq > ifreq)
       maxFreq = ifreq;
@@ -1465,9 +1663,26 @@ void TrackArtist::DrawClipSpectrum(WaveTrack* track, WaveClip *clip,
       minFreq = 0;
       gPrefs->Write(wxT("/Spectrum/MinFreq"), 0L);
    }
+#endif
+#ifdef EXPERIMENTAL_FIND_NOTES
+   bool fftFindNotes = (gPrefs->Read(wxT("/Spectrum/FFTFindNotes"), 0L) != 0);
+   int findNotesMinA = gPrefs->Read(wxT("/Spectrum/FindNotesMinA"), -30.0);
+   int numberOfMaxima = gPrefs->Read(wxT("/Spectrum/FindNotesN"), 5L);
+   bool findNotesQuantize = (gPrefs->Read(wxT("/Spectrum/FindNotesQuantize"), 0L) != 0);
+#endif //EXPERIMENTAL_FIND_NOTES
    bool usePxCache = false;
 
-   if( !updated && clip->mSpecPxCache->valid && (clip->mSpecPxCache->len == mid.height * mid.width) ) {
+   if( !updated && clip->mSpecPxCache->valid && (clip->mSpecPxCache->len == mid.height * mid.width) 
+#ifdef EXPERIMENTAL_FFT_Y_GRID
+   && fftYGrid==fftYGridOld
+#endif //EXPERIMENTAL_FFT_Y_GRID
+#ifdef EXPERIMENTAL_FIND_NOTES
+   && fftFindNotes==fftFindNotesOld
+   && findNotesMinA==findNotesMinAOld
+   && numberOfMaxima==findNotesNOld
+   && findNotesQuantize==findNotesQuantizeOld
+#endif
+   ) {
       usePxCache = true;
    }
    else {
@@ -1475,6 +1690,12 @@ void TrackArtist::DrawClipSpectrum(WaveTrack* track, WaveClip *clip,
       clip->mSpecPxCache = new SpecPxCache(mid.width * mid.height);
       usePxCache = false;
       clip->mSpecPxCache->valid = true;
+#ifdef EXPERIMENTAL_FIND_NOTES
+      fftFindNotesOld=fftFindNotes;
+      findNotesMinAOld=findNotesMinA;
+      findNotesNOld=numberOfMaxima;
+      findNotesQuantizeOld=findNotesQuantize;
+#endif
    }
 
    int minSamples = int (minFreq * windowSize / rate + 0.5);   // units are fft bins
@@ -1483,10 +1704,265 @@ void TrackArtist::DrawClipSpectrum(WaveTrack* track, WaveClip *clip,
 
    int x = 0;
    sampleCount w1 = (sampleCount) ((t0*rate + x *rate *tstep) + .5);
-   while (x < mid.width) {
+ 
+#ifdef LOGARITHMIC_SPECTRUM
+   const float 
+      e=exp(1.0f), 
+      log2=log(2.0f),
+      f=rate/2.0f/half, 
+      lmin=log(float(minFreq)),
+      lmax=log(float(maxFreq)),
+#ifdef EXPERIMENTAL_FFT_SKIP_POINTS
+      lmins=log(float(minFreq)/(fftSkipPoints+1)),
+      lmaxs=log(float(maxFreq)/(fftSkipPoints+1)),
+#else //!EXPERIMENTAL_FFT_SKIP_POINTS
+      lmins=lmin,
+      lmaxs=lmax,
+#endif //EXPERIMENTAL_FFT_SKIP_POINTS
+      scale=lmax-lmin, 
+      scale2=(lmax-lmin)/log2, 
+      lmin2=lmin/log2,
+      expo=exp(scale);
+
+#ifdef EXPERIMENTAL_FFT_Y_GRID
+   bool *yGrid;
+   yGrid=new bool[mid.height];
+   for (int y = 0; y < mid.height; y++) {
+      float n =(float(y  )/mid.height*scale2-lmin2)*12; 
+      float n2=(float(y+1)/mid.height*scale2-lmin2)*12; 
+      float f =float(minFreq)/(fftSkipPoints+1)*pow(2.0f, n /12.0f+lmin2);
+      float f2=float(minFreq)/(fftSkipPoints+1)*pow(2.0f, n2/12.0f+lmin2);
+      n =log(f /440)/log2*12;
+      n2=log(f2/440)/log2*12;
+      if (floor(n) < floor(n2))
+         yGrid[y]=true;
+      else
+         yGrid[y]=false;
+   }
+#endif //EXPERIMENTAL_FFT_Y_GRID
+#endif //LOGARITHMIC_SPECTRUM
+
+#ifdef EXPERIMENTAL_FIND_NOTES
+   int maxima[128];
+   float maxima0[128], maxima1[128];
+   const float 
+#ifdef EXPERIMENTAL_FFT_SKIP_POINTS
+      f2bin = half/(rate/2.0f/(fftSkipPoints+1)),
+#else //!EXPERIMENTAL_FFT_SKIP_POINTS
+      f2bin = half/(rate/2.0f),
+#endif //EXPERIMENTAL_FFT_SKIP_POINTS
+      bin2f = 1.0f/f2bin,
+      minDistance = pow(2.0f, 2.0f/12.0f),
+      i0=exp(lmin)/f,
+      i1=exp(scale+lmin)/f,
+      minColor=0.0f;
+   const int maxTableSize=1024;
+   int *indexes=new int[maxTableSize];
+#endif //EXPERIMENTAL_FIND_NOTES
+
+   while (x < mid.width) 
+   {
       sampleCount w0 = w1;
       w1 = (sampleCount) ((t0*rate + (x+1) *rate *tstep) + .5);
 
+#ifdef LOGARITHMIC_SPECTRUM
+      if (!logF)
+      {
+         for (int yy = 0; yy < mid.height; yy++) {
+            bool selflag = (ssel0 <= w0 && w1 < ssel1);
+            unsigned char rv, gv, bv;
+            float value;
+
+            if(!usePxCache) {
+               float bin0 = float (yy) * binPerPx + minSamples;
+               float bin1 = float (yy + 1) * binPerPx + minSamples;
+
+
+               if (int (bin1) == int (bin0))
+                  value = freq[half * x + int (bin0)];
+               else {
+                  float binwidth= bin1 - bin0;
+                  value = freq[half * x + int (bin0)] * (1.f - bin0 + (int)bin0);
+
+                  bin0 = 1 + int (bin0);
+                  while (bin0 < int (bin1)) {
+                     value += freq[half * x + int (bin0)];
+                     bin0 += 1.0;
+                  }
+                  value += freq[half * x + int (bin1)] * (bin1 - int (bin1));
+
+                  value /= binwidth;
+               }
+
+               if (!autocorrelation) {
+                  // Last step converts dB to a 0.0-1.0 range
+                  value = (value + 80.0) / 80.0;
+               }
+
+               if (value > 1.0)
+                  value = float(1.0);
+               if (value < 0.0)
+                  value = float(0.0);
+               clip->mSpecPxCache->values[x * mid.height + yy] = value;
+            }
+            else
+               value = clip->mSpecPxCache->values[x * mid.height + yy];
+
+            GetColorGradient(value, selflag, isGrayscale, &rv, &gv, &bv);
+
+            int px = ((mid.height - 1 - yy) * mid.width + x) * 3;
+            data[px++] = rv;
+            data[px++] = gv;
+            data[px] = bv;
+         }
+      }
+      else //logF
+      {
+         bool selflag = (ssel0 <= w0 && w1 < ssel1);
+         unsigned char rv, gv, bv;
+         float value;
+         int x0=x*half;
+
+#ifdef EXPERIMENTAL_FIND_NOTES
+         int maximas=0;
+         if (!usePxCache && fftFindNotes) {
+            for (int i = maxTableSize-1; i >= 0; i--)
+               indexes[i]=-1;
+            
+            // Build a table of (most) values, put the index in it.
+            for (int i = int(i0); i < int(i1); i++) {
+               float freqi=freq[x0+int(i)];
+               int value=int((freqi+80.0f)/80.0f*(maxTableSize-1));
+               if (value < 0)
+                  value=0;
+               if (value >= maxTableSize)
+                  value=maxTableSize-1;
+               indexes[value]=i;
+            }
+            // Build from the indices an array of maxima.
+            for (int i = maxTableSize-1; i >= 0; i--) {
+               int index=indexes[i];
+               if (index >= 0) {
+                  float freqi=freq[x0+index];
+                  if (freqi < findNotesMinA) 
+                     break;
+
+                  bool ok=true;
+                  for (int m=0; m < maximas; m++) {
+                     // Avoid to store very close maxima.
+                     float maxm = maxima[m];
+                     if (maxm/index < minDistance && index/maxm < minDistance) {
+                        ok=false;
+                        break;
+                     }
+                  }
+                  if (ok) {
+                     maxima[maximas++] = index;
+                     if (maximas >= numberOfMaxima)
+                        break;
+                  }
+               }
+            }
+
+// The f2pix helper macro converts a frequency into a pixel coordinate.
+#define f2pix(f) (log(f)-lmins)/(lmaxs-lmins)*mid.height
+
+            // Possibly quantize the maxima frequencies and create the pixel block limits.
+            for (int i=0; i < maximas; i++) {
+               int index=maxima[i];
+               float f = float(index)*bin2f;
+               if (findNotesQuantize)
+               {  f = exp(int(log(f/440)/log2*12-0.5)/12.0f*log2)*440;
+                  maxima[i] = f*f2bin;
+               }
+               float f0 = exp((log(f/440)/log2*24-1)/24.0f*log2)*440;
+               maxima0[i] = f2pix(f0);
+               float f1 = exp((log(f/440)/log2*24+1)/24.0f*log2)*440;
+               maxima1[i] = f2pix(f1);
+            }
+         }
+         int it=0;
+         int oldBin0=-1;
+         bool inMaximum = false;
+#endif //EXPERIMENTAL_FIND_NOTES
+
+         for (int yy = 0; yy < mid.height; yy++) {
+            if(!usePxCache) {
+               float h=float(yy)/mid.height; 
+               float yy2=exp(h*scale+lmin)/f;
+               if (int(yy2)>=half)
+                  yy2=half-1;
+               if (yy2<0)
+                  yy2=0;
+               float bin0 = float(yy2);
+               float h1=float(yy+1)/mid.height; 
+               float yy3=exp(h1*scale+lmin)/f;
+               if (int(yy3)>=half)
+                  yy3=half-1;
+               if (yy3<0)
+                  yy3=0;
+               float bin1 = float(yy3);
+
+#ifdef EXPERIMENTAL_FIND_NOTES
+               if (fftFindNotes) {
+                  if (it < maximas) {
+                     float i0=maxima0[it];
+                     if (yy >= i0)
+                        inMaximum = true;
+
+                     if (inMaximum) {
+                        float i1=maxima1[it];
+                        if (yy+1 <= i1) {
+                           value=sumFreqValues(freq, x0, bin0, bin1);
+                           if (value < findNotesMinA)
+                              value = minColor;
+                           else
+                              value = (value + 80.0) / 80.0;
+                        } else {
+                           it++;
+                           inMaximum = false;
+                           value = minColor;
+                        }
+                     } else {
+                        value = minColor;
+                     }
+                  } else
+                     value = minColor;
+               } else
+#endif //EXPERIMENTAL_FIND_NOTES
+               {
+                  value=sumFreqValues(freq, x0, bin0, bin1);
+                  if (!autocorrelation) {
+                     // Last step converts dB to a 0.0-1.0 range
+                     value = (value + 80.0) / 80.0;
+                  }
+               }
+               if (value > 1.0)
+                  value = float(1.0);
+               if (value < 0.0)
+                  value = float(0.0);
+               clip->mSpecPxCache->values[x * mid.height + yy] = value;
+            }
+            else
+               value = clip->mSpecPxCache->values[x * mid.height + yy];
+
+            GetColorGradient(value, selflag, isGrayscale, &rv, &gv, &bv);
+
+#ifdef EXPERIMENTAL_FFT_Y_GRID
+            if (fftYGrid && yGrid[yy]) {
+               rv /= 1.1f;
+               gv /= 1.1f;
+               bv /= 1.1f;
+            }
+#endif //EXPERIMENTAL_FFT_Y_GRID
+
+            int px = ((mid.height - 1 - yy) * mid.width + x) * 3;
+            data[px++] = rv;
+            data[px++] = gv;
+            data[px] = bv;
+         }
+      }
+#else //!LOGARITHMIC_SPECTRUM
       for (int yy = 0; yy < mid.height; yy++) {
          bool selflag = (ssel0 <= w0 && w1 < ssel1);
          unsigned char rv, gv, bv;
@@ -1534,6 +2010,7 @@ void TrackArtist::DrawClipSpectrum(WaveTrack* track, WaveClip *clip,
          data[px++] = gv;
          data[px] = bv;
       }
+#endif //LOGARITHMIC_SPECTRUM
       x++;
    }
 
@@ -1552,6 +2029,12 @@ void TrackArtist::DrawClipSpectrum(WaveTrack* track, WaveClip *clip,
    delete image;
    delete[] where;
    delete[] freq;
+#ifdef EXPERIMENTAL_FFT_Y_GRID
+   delete[] yGrid;
+#endif //EXPERIMENTAL_FFT_Y_GRID
+#ifdef EXPERIMENTAL_FIND_NOTES
+   delete[] indexes;
+#endif //EXPERIMENTAL_FIND_NOTES
 }
 
 /*
@@ -1744,7 +2227,7 @@ void TrackArtist::DrawNoteTrack(NoteTrack *track,
   int visibleChannels = track->mVisibleChannels;
 
   if (!track->GetSelected())
-	sel0 = sel1 = 0.0;
+   sel0 = sel1 = 0.0;
 
   int ctrpitch = 60;
   int pitch0;
@@ -2013,11 +2496,11 @@ void TrackArtist::DrawNoteTrack(NoteTrack *track,
               // now do justification
               const char *s = LookupStringAttribute(note, texts, "");
               #ifdef __WXMAC__
-		      long textWidth, textHeight;
+            long textWidth, textHeight;
               #else
-		      int textWidth, textHeight;
+            int textWidth, textHeight;
               #endif
-		      dc.GetTextExtent(LAT1CTOWX(s), &textWidth, &textHeight);
+            dc.GetTextExtent(LAT1CTOWX(s), &textWidth, &textHeight);
               long hoffset = 0;
               long voffset = -textHeight; // default should be baseline of text
 
@@ -2072,9 +2555,14 @@ void TrackArtist::DrawTimeTrack(TimeTrack *track,
    wxRect envRect = r;
    envRect.height -= 2;
    track->GetEnvelope()->Draw(dc, envRect, viewInfo->h, viewInfo->zoom, 
-			      false,0.0,1.0);
+               false,0.0,1.0);
 }
 
+void TrackArtist::UpdatePrefs()
+{
+   mdBrange = gPrefs->Read(wxT("/GUI/EnvdBRange"), mdBrange);
+   mShowClipping = gPrefs->Read(wxT("/GUI/ShowClipping"), mShowClipping);
+}
 
 // Indentation settings for Vim and Emacs and unique identifier for Arch, a
 // version control system. Please do not modify past this point.
