@@ -43,19 +43,6 @@ class TrackPanelAx;
 
 struct ViewInfo;
 
-#ifdef OLD_CODE
-/// Holds a bitmap and its position, this is an element of a tpBitmapArray,
-/// which appears to be not used anymore.
-struct tpBitmap
-{
-   wxBitmap *bitmap;
-   wxCoord x;
-   wxCoord y;
-};
-
-WX_DEFINE_ARRAY(tpBitmap *, tpBitmapArray);
-#endif
-
 WX_DEFINE_ARRAY(LWSlider *, LWSliderArray);
 
 class TrackClip
@@ -98,16 +85,25 @@ public:
    TrackInfo(wxWindow * pParentIn);
    ~TrackInfo();
 
+#ifdef EXPERIMENTAL_RULER_AUTOSIZE
+   int GetTitleWidth() const;
+#else //!EXPERIMENTAL_RULER_AUTOSIZE
    int GetTitleWidth() const { return 100; }
+#endif //EXPERIMENTAL_RULER_AUTOSIZE
+
 private:
    void MakeMoreSliders();
    void EnsureSufficientSliders(int index);
 
+#ifdef EXPERIMENTAL_RULER_AUTOSIZE
+   void DrawBackground(wxDC * dc, const wxRect r, bool bSelected, bool bHasMuteSolo, const int labelw, const int vrul);
+#else //!EXPERIMENTAL_RULER_AUTOSIZE
    void DrawBackground(wxDC * dc, const wxRect r, bool bSelected, bool bHasMuteSolo, const int labelw);
+#endif //EXPERIMENTAL_RULER_AUTOSIZE
    void DrawBordersWithin(wxDC * dc, const wxRect r, bool bHasMuteSolo );
    void DrawCloseBox(wxDC * dc, const wxRect r, bool down);
    void DrawTitleBar(wxDC * dc, const wxRect r, Track * t, bool down);
-   void DrawMuteSolo(wxDC * dc, const wxRect r, Track * t, bool down, bool solo);
+   void DrawMuteSolo(wxDC * dc, const wxRect r, Track * t, bool down, bool solo, bool bHasSoloButton);
    void DrawVRuler(wxDC * dc, const wxRect r, Track * t);
    void DrawSliders(wxDC *dc, WaveTrack *t, wxRect r, int index);
    void DrawMinimize(wxDC * dc, const wxRect r, Track * t, bool down, bool minimized);
@@ -115,7 +111,7 @@ private:
    void GetTrackControlsRect(const wxRect r, wxRect &dest) const;
    void GetCloseBoxRect(const wxRect r, wxRect &dest) const;
    void GetTitleBarRect(const wxRect r, wxRect &dest) const;
-   void GetMuteSoloRect(const wxRect r, wxRect &dest, bool solo) const;
+   void GetMuteSoloRect(const wxRect r, wxRect &dest, bool solo, bool bHasSoloButton) const;
    void GetGainRect(const wxRect r, wxRect &dest) const;
    void GetPanRect(const wxRect r, wxRect &dest) const;
    void GetMinimizeRect(const wxRect r, wxRect &dest, bool minimized) const;
@@ -148,6 +144,7 @@ class TrackPanel:public wxPanel {
 
    virtual ~ TrackPanel();
 
+   void BuildMenus(void);
    void UpdatePrefs();
 
    void OnSize(wxSizeEvent & event);
@@ -217,6 +214,9 @@ class TrackPanel:public wxPanel {
    void SetFocusedTrack(Track *t);
 
    void HandleCursorForLastMouseEvent();
+#ifdef EXPERIMENTAL_RULER_AUTOSIZE
+   void UpdateVRulerRect();
+#endif //EXPERIMENTAL_RULER_AUTOSIZE
 
  private:
 
@@ -361,7 +361,13 @@ class TrackPanel:public wxPanel {
 
 //   int GetTitleWidth() const { return 100; }
    int GetTitleOffset() const { return 0; }
+
+#ifdef EXPERIMENTAL_RULER_AUTOSIZE
+   int GetVRulerWidth() const;
+#else //!EXPERIMENTAL_RULER_AUTOSIZE
    int GetVRulerWidth() const { return 36;}
+#endif //EXPERIMENTAL_RULER_AUTOSIZE
+
    int GetVRulerOffset() const { return GetTitleOffset() + mTrackInfo.GetTitleWidth();}
    int GetLabelWidth() const { return mTrackInfo.GetTitleWidth() + GetVRulerWidth();}
 
@@ -383,6 +389,10 @@ private:
    int IdOfRate( int rate );
    int IdOfFormat( int format );
 
+   // Accessors...
+   bool IsSimpleSolo(){ return mSoloPref==wxT("Simple");};
+   bool HasSoloButton(){  return mSoloPref!=wxT("None");};
+
    //JKC: These two belong in the label track.
    int mLabelTrackStartXPos;
    int mLabelTrackStartYPos;
@@ -565,12 +575,17 @@ private:
 
    TrackPanelAx *mAx;
 
+   wxString mSoloPref;
+
  private:
 
    // The screenshot class needs to access internals
    friend class ScreenFrame;
 
  public:
+#ifdef EXPERIMENTAL_RULER_AUTOSIZE
+   wxSize vrulerSize;
+#endif //EXPERIMENTAL_RULER_AUTOSIZE
 
    DECLARE_EVENT_TABLE()
 };
