@@ -70,7 +70,11 @@ WaveTrack::WaveTrack(DirManager *projDirManager, sampleFormat format, double rat
       rate = GetActiveProject()->GetRate();
    }
 
+#ifdef EXPERIMENTAL_SAVE_DEFAULT_VIEW
+   gPrefs->Read(wxT("/GUI/DefaultViewMode"), &mDisplay, 0L);
+#else //!EXPERIMENTAL_SAVE_DEFAULT_VIEW
    mDisplay = 0; // Move to GUIWaveTrack
+#endif //EXPERIMENTAL_SAVE_DEFAULT_VIEW
 
    mLegacyProjectFileOffset = 0;
 
@@ -90,7 +94,11 @@ WaveTrack::WaveTrack(DirManager *projDirManager, sampleFormat format, double rat
 WaveTrack::WaveTrack(WaveTrack &orig):
    Track(orig)
 {
+#ifdef EXPERIMENTAL_SAVE_DEFAULT_VIEW
+   gPrefs->Read(wxT("/GUI/DefaultViewMode"), &mDisplay, 0L);
+#else //!EXPERIMENTAL_SAVE_DEFAULT_VIEW
    mDisplay = 0; // Move to GUIWaveTrack
+#endif //EXPERIMENTAL_SAVE_DEFAULT_VIEW
 
    mLegacyProjectFileOffset = 0;
 
@@ -680,13 +688,25 @@ bool WaveTrack::HandleClear(double t0, double t1,
                }
             }
             else {
-               // frame clip envelope points here
+               /* We are going to delete part of the clip here. The clip may
+                * have envelope points, and we need to ensure that the envelope
+                * outside of the cleared region is not affected. This means
+                * putting in "glue" points where the clip enters and leaves the
+                * region being cleared. If one of the ends of the clip is inside
+                * the region, then one of the glue points will be redundant. */
                // clip->Clear keeps points < t0 and >= t1 via Envelope::CollapseRegion
                double val;
-               val = clip->GetEnvelope()->GetValue(t0);
-               clip->GetEnvelope()->Insert(t0 - clip->GetOffset() - 1.0/clip->GetRate(), val);
-               val = clip->GetEnvelope()->GetValue(t1);
-               clip->GetEnvelope()->Insert(t1 - clip->GetOffset(), val);
+               if (t0 > clip->GetStartTime())
+                  {  // start of clip is before start of region to clear
+                  val = clip->GetEnvelope()->GetValue(t0);
+                  clip->GetEnvelope()->Insert(t0 - clip->GetOffset() - 1.0/clip->GetRate(), val);
+                  }
+
+               if (t1 < clip->GetEndTime())
+                  {  // end of clip is after end of region
+                  val = clip->GetEnvelope()->GetValue(t1);
+                  clip->GetEnvelope()->Insert(t1 - clip->GetOffset(), val);
+                  }
 
                if (!clip->Clear(t0,t1))
                   return false;
@@ -1384,6 +1404,19 @@ Envelope* WaveTrack::GetEnvelopeAtX(int xcoord)
       return NULL;
 }
 
+// Search for any active DragPoint on the current track
+Envelope* WaveTrack::GetActiveEnvelope(void)
+{
+   for (WaveClipList::Node* it=GetClipIterator(); it; it=it->GetNext())
+   {
+      WaveClip* clip = it->GetData();
+      Envelope* env = clip->GetEnvelope() ;
+      if (env->GetDragPoint() >= 0)
+         return env;
+   }
+   return NULL;
+}
+
 Sequence* WaveTrack::GetSequenceAtX(int xcoord)
 {
    WaveClip* clip = GetClipAtX(xcoord);
