@@ -23,6 +23,7 @@
 
 #include "ChangeSpeed.h"
 
+#include "../ShuttleGui.h"
 #include "../Envelope.h"
 #include "../Prefs.h"
 
@@ -87,41 +88,49 @@ bool EffectChangeSpeed::Process()
 	// Similar to EffectSoundTouch::Process()
 
    //Iterate over each track
-   TrackListIterator iter(mWaveTracks);
-   WaveTrack *track = (WaveTrack *) iter.First();
+   this->CopyInputWaveTracks(); // Set up m_pOutputWaveTracks.
+   bool bGoodResult = true;
+
+   TrackListIterator iter(m_pOutputWaveTracks);
+   WaveTrack* pOutWaveTrack = (WaveTrack*)(iter.First());
    mCurTrackNum = 0;
 	m_maxNewLength = 0.0;
-	double curT0;
-	double curT1;
-   while (track) {
+   while (pOutWaveTrack != NULL)
+   {
       //Get start and end times from track
-      double trackStart = track->GetStartTime();
-      double trackEnd = track->GetEndTime();
+      mCurT0 = pOutWaveTrack->GetStartTime();
+      mCurT1 = pOutWaveTrack->GetEndTime();
 
       //Set the current bounds to whichever left marker is
       //greater and whichever right marker is less:
-      curT0 = mT0 < trackStart? trackStart: mT0;
-      curT1 = mT1 > trackEnd? trackEnd: mT1;
+      mCurT0 = wxMax(mT0, mCurT0);
+      mCurT1 = wxMin(mT1, mCurT1);
 
       // Process only if the right marker is to the right of the left marker
-      if (curT1 > curT0) {
+      if (mCurT1 > mCurT0) {
 
          //Transform the marker timepoints to samples
-         longSampleCount start = track->TimeToLongSamples(curT0);
-         longSampleCount end = track->TimeToLongSamples(curT1);
+         longSampleCount start = pOutWaveTrack->TimeToLongSamples(mCurT0);
+         longSampleCount end = pOutWaveTrack->TimeToLongSamples(mCurT1);
 
          //ProcessOne() (implemented below) processes a single track
-         if (!ProcessOne(track, start, end))
-            return false;
+         if (!ProcessOne(pOutWaveTrack, start, end))
+         {
+            bGoodResult = false;
+            break;
+         }
       }
       
       //Iterate to the next track
-      track = (WaveTrack *) iter.Next();
+      pOutWaveTrack = (WaveTrack*)(iter.Next());
       mCurTrackNum++;
    }
 
-	mT1 = mT0 + m_maxNewLength; // Update selection.
-   return true;
+   this->ReplaceProcessedWaveTracks(bGoodResult); 
+
+// mT1 = mT0 + m_maxNewLength; // Update selection.
+
+   return bGoodResult;
 }
 
 // ProcessOne() takes a track, transforms it to bunch of buffer-blocks,
@@ -210,8 +219,8 @@ bool EffectChangeSpeed::ProcessOne(WaveTrack * track,
    // Take the output track and insert it in place of the original
    // sample data
 	if (bLoopSuccess) {
-		track->Clear(mT0, mT1);
-		track->Paste(mT0, outputTrack);
+		track->Clear(mCurT0, mCurT1);
+		track->Paste(mCurT0, outputTrack);
 	}
 
 	double newLength = outputTrack->GetEndTime(); 
@@ -246,8 +255,7 @@ enum {
    ID_TEXT_PERCENTCHANGE = 10001,
    ID_SLIDER_PERCENTCHANGE,
    ID_CHOICE_FROMVINYL,
-   ID_CHOICE_TOVINYL,
-	ID_BUTTON_PREVIEW
+   ID_CHOICE_TOVINYL
 };
 
 
@@ -262,7 +270,7 @@ BEGIN_EVENT_TABLE(ChangeSpeedDialog, wxDialog)
     EVT_CHOICE(ID_CHOICE_FROMVINYL, ChangeSpeedDialog::OnChoice_FromVinyl)
     EVT_CHOICE(ID_CHOICE_TOVINYL, ChangeSpeedDialog::OnChoice_ToVinyl)
 
-    EVT_BUTTON(ID_BUTTON_PREVIEW, ChangeSpeedDialog::OnPreview)
+    EVT_BUTTON(ID_EFFECT_PREVIEW, ChangeSpeedDialog::OnPreview)
 END_EVENT_TABLE()
 
 ChangeSpeedDialog::ChangeSpeedDialog(EffectChangeSpeed * effect,
@@ -328,7 +336,7 @@ ChangeSpeedDialog::ChangeSpeedDialog(EffectChangeSpeed * effect,
 	//v Override wxTextValidator to disallow negative values <= -100.0?
    m_pTextCtrl_PercentChange = 
 		new wxTextCtrl(this, ID_TEXT_PERCENTCHANGE, wxT("0.0"), 
-							wxDefaultPosition, wxSize(40, -1), 0,
+							wxDefaultPosition, wxSize(60, -1), 0,
 							wxTextValidator(wxFILTER_NUMERIC));
    pBoxSizer_PercentChange->Add(m_pTextCtrl_PercentChange, 0, 
 											wxALIGN_CENTER_VERTICAL | wxALIGN_LEFT | wxALL, 4);
@@ -363,7 +371,7 @@ ChangeSpeedDialog::ChangeSpeedDialog(EffectChangeSpeed * effect,
 
    m_pChoice_FromVinyl = 
 		new wxChoice(this, ID_CHOICE_FROMVINYL, 
-							wxDefaultPosition, wxSize(64, -1), numChoices, strArray_VinylRPM);
+							wxDefaultPosition, wxDefaultSize, numChoices, strArray_VinylRPM);
    pBoxSizer_Vinyl->Add(m_pChoice_FromVinyl, 0, wxALIGN_CENTER | wxALL, 4);
 
    pStaticText = new wxStaticText(this, -1, _("to"),
@@ -373,38 +381,20 @@ ChangeSpeedDialog::ChangeSpeedDialog(EffectChangeSpeed * effect,
 
    m_pChoice_ToVinyl = 
 		new wxChoice(this, ID_CHOICE_TOVINYL, 
-							wxDefaultPosition, wxSize(64, -1), numChoices, strArray_VinylRPM);
+							wxDefaultPosition, wxDefaultSize, numChoices, strArray_VinylRPM);
    pBoxSizer_Vinyl->Add(m_pChoice_ToVinyl, 0, wxALIGN_CENTER | wxALL, 4);
 
    pBoxSizer_Dialog->Add(pBoxSizer_Vinyl, 0, wxALIGN_CENTER | wxALL, 4);
 
-
-	// Preview, OK, & Cancel buttons
-   pBoxSizer_Dialog->Add(0, 8, 0); // spacer
-
-   wxBoxSizer * pBoxSizer_OK = new wxBoxSizer(wxHORIZONTAL);
-
-   wxButton * pButton_Preview = 
-		new wxButton(this, ID_BUTTON_PREVIEW, m_pEffect->GetPreviewName());
-   pBoxSizer_OK->Add(pButton_Preview, 0, wxALIGN_CENTER | wxALL, 4);
-   pBoxSizer_OK->Add(32, 8); // horizontal spacer
-
-   wxButton * pButton_Cancel =
-       new wxButton(this, wxID_CANCEL, _("&Cancel"), wxDefaultPosition, wxDefaultSize, 0);
-   pBoxSizer_OK->Add(pButton_Cancel, 0, wxALIGN_CENTER | wxALL, 4);
-
-   wxButton * pButton_OK =
-       new wxButton(this, wxID_OK, _("&OK"), wxDefaultPosition, wxDefaultSize, 0);
-   pButton_OK->SetDefault();
-   pBoxSizer_OK->Add(pButton_OK, 0, wxALIGN_CENTER | wxALL, 4);
-
-   pBoxSizer_Dialog->Add(pBoxSizer_OK, 0, wxALIGN_CENTER | wxALL, 8);
-
+   // Preview, OK, & Cancel buttons
+   pBoxSizer_Dialog->Add(CreateStdButtonSizer(this, ePreviewButton|eCancelButton|eOkButton), 0, wxEXPAND);
 
    this->SetAutoLayout(true);
    this->SetSizer(pBoxSizer_Dialog);
    pBoxSizer_Dialog->Fit(this);
    pBoxSizer_Dialog->SetSizeHints(this);
+
+   effect->SetDialog(this);
 }
 
 bool ChangeSpeedDialog::Validate()
