@@ -64,10 +64,10 @@ bool EffectDtmf::PromptUser()
       gPrefs->Read(wxT("/CsPresets/DtmfGen_SequenceDuration"), &dtmfDuration, 1L);
       dlog.dIsSelection = false;
    }
-
+   /// \todo this code shouldn't be using /CsPresets - need to review its use
    gPrefs->Read(wxT("/CsPresets/DtmfGen_String"), &dtmfString, wxT("audacity"));
    gPrefs->Read(wxT("/CsPresets/DtmfGen_DutyCycle"), &dtmfDutyCycle, 550L);
-   gPrefs->Read(wxT("/CsPresets/DtmfGen_Amplitude"), &dtmfAmplitude, 1);
+   gPrefs->Read(wxT("/CsPresets/DtmfGen_Amplitude"), &dtmfAmplitude, 0.8f);
 
    dtmfNTones = wxStrlen(dtmfString);
 
@@ -427,6 +427,8 @@ DtmfDialog::DtmfDialog(wxWindow * parent, const wxString & title): EffectDialog(
    dTone = 0;
    dSilence = 0;
    dDuration = 0;
+
+   mDtmfDurationT = NULL;
 }
 
 void DtmfDialog::PopulateOrExchange( ShuttleGui & S )
@@ -443,28 +445,30 @@ void DtmfDialog::PopulateOrExchange( ShuttleGui & S )
 
       S.TieTextBox(_("Amplitude (0-1)"),  dAmplitude, 10);
 
-      S.AddPrompt(_("DTMF duration:"));
-      mDtmfDurationT = new
-         TimeTextCtrl(this,
-                      ID_DTMF_DURATION_TEXT,
-                      /*
-                      use this instead of "seconds" because if a selection is passed to the effect,
-                      I want it (dDuration) to be used as the duration, and with "seconds" this does
-                      not always work properly. For example, it rounds down to zero...
-                      */
-                      TimeTextCtrl::GetBuiltinFormat(dIsSelection==true?(wxT("hh:mm:ss + samples")):(wxT("seconds"))),
-                      dDuration,
-                      44100,
-                      wxDefaultPosition,
-                      wxDefaultSize,
-                      true);
+      S.AddPrompt(_("Duration:"));
+      if (mDtmfDurationT == NULL)
+      {
+         mDtmfDurationT = new
+            TimeTextCtrl(this,
+                         ID_DTMF_DURATION_TEXT,
+                         wxT(""),
+                         dDuration,
+                         44100,
+                         wxDefaultPosition,
+                         wxDefaultSize,
+                         true);
+         /* use this instead of "seconds" because if a selection is passed to the
+         * effect, I want it (dDuration) to be used as the duration, and with
+         * "seconds" this does not always work properly. For example, it rounds
+         * down to zero... */
+         mDtmfDurationT->SetFormatString(mDtmfDurationT->GetBuiltinFormat(dIsSelection==true?(wxT("hh:mm:ss + samples")):(wxT("seconds"))));
+         mDtmfDurationT->EnableMenu();
+      }
       S.AddWindow(mDtmfDurationT);
-      mDtmfDurationT->EnableMenu();
 
       S.AddFixedText(_("Tone/silence ratio:"), false);
       S.SetStyle(wxSL_HORIZONTAL | wxEXPAND);
-      mDtmfDutyS = S.Id(ID_DTMF_DUTYCYCLE_SLIDER).AddSlider(wxT(""), (int)dDutyCycle, DUTY_MAX);
-      mDtmfDutyS->SetRange(DUTY_MIN, DUTY_MAX);
+      mDtmfDutyS = S.Id(ID_DTMF_DUTYCYCLE_SLIDER).AddSlider(wxT(""), (int)dDutyCycle, DUTY_MAX, DUTY_MIN);
 
       S.SetSizeHints(-1,-1);
    }
