@@ -206,7 +206,7 @@ bool ExportPlugin::Export(AudacityProject *project,
                           double t0,
                           double t1,
                           MixerSpec *mixerSpec,
-                          bool use_meta)
+                          Tags *metadata)
 {
    if (project == NULL) {
       project = GetActiveProject();
@@ -293,6 +293,12 @@ bool Exporter::Process(AudacityProject *project, bool selectedOnly, double t0, d
       return false;
    }
 
+   // Let user edit MetaData 
+
+   if (!(project->GetTags()->ShowEditDialog(project, _("Edit Metadata"), mProject->GetShowId3Dialog()))) {
+      return false;
+   }
+
    // Check for down mixing
    if (!CheckMix()) {
       return false;
@@ -930,6 +936,7 @@ ExportMixerDialog::ExportMixerDialog( TrackList *tracks, bool selectedOnly,
    TrackListIterator iter( tracks );
    
    for( Track *t = iter.First(); t; t = iter.Next() )
+   {
       if( t->GetKind() == Track::Wave && ( t->GetSelected() || !selectedOnly ) )
       {
          numTracks++;
@@ -943,7 +950,15 @@ ExportMixerDialog::ExportMixerDialog( TrackList *tracks, bool selectedOnly,
          else
             mTrackNames.Add( t->GetName() );
       }
+   }
 
+   // JKC: This is an attempt to fix a 'watching brief' issue, where the slider is
+   // sometimes not slidable.  My suspicion is that a mixer may incorrectly
+   // state the number of channels - so we assume there are always at least two.
+   // The downside is that if someone is exporting to a mono device, the dialog
+   // will allow them to output to two channels. Hmm.  We may need to revisit this.
+   if (maxNumChannels < 2 )
+      maxNumChannels = 2;
    if (maxNumChannels > 32)
       maxNumChannels = 32;
 
