@@ -30,6 +30,7 @@
 #include <wx/event.h>
 #include <wx/filedlg.h>
 #include <wx/filefn.h>
+#include <wx/filename.h>
 #include <wx/intl.h>
 #include <wx/msgdlg.h>
 #include <wx/radiobut.h>
@@ -37,6 +38,7 @@
 #include <wx/statbox.h>
 #include <wx/stattext.h>
 #include <wx/textctrl.h>
+#include <wx/textdlg.h>
 
 #include "Export.h"
 #include "ExportMultiple.h"
@@ -48,6 +50,11 @@
 #include "../Prefs.h"
 #include "../Tags.h"
 
+
+/* define our dynamic array of export settings */
+#include <wx/arrimpl.cpp>     // much hackery
+WX_DEFINE_OBJARRAY( ExportKitArray )
+
 enum {
    FormatID = 10001,
    OptionsID,
@@ -91,6 +98,11 @@ ExportMultiple::ExportMultiple(AudacityProject *project)
    mTracks = project->GetTracks();
    mPlugins = mExporter.GetPlugins();
 
+   // create array of characters not allowed in file names
+   wxString forbid = wxFileName::GetForbiddenChars();
+   for(unsigned int i=0; i < forbid.Length(); i++)
+      exclude.Add( forbid.Mid(i, 1) );
+
    ShuttleGui S(this, eIsCreatingFromPrefs);
 
    // Creating some of the widgets cause cause events to fire
@@ -425,28 +437,6 @@ void ExportMultiple::OnExport(wxCommandEvent& event)
 
    mFormatIndex = mFormat->GetSelection();
    bool overwrite = mOverwrite->GetValue();
-
-   if (mPlugins[mFormatIndex]->GetCanMetaData()) {
-      Tags *tags = mProject->GetTags();
-
-      if (tags->IsEmpty()) {
-         tags->SetTag(TAG_TITLE, wxT("(automatic)"));
-         tags->SetTag(TAG_TRACK, wxT("0"));
-         tags->AllowEditTitle(false);
-         tags->AllowEditTrackNumber(false);
-         bool rval = tags->ShowEditDialog(mProject,
-                                          _("Edit the metadata"),
-                                          true);
-         tags->SetTag(TAG_TITLE, wxT(""));
-         tags->SetTag(TAG_TRACK, wxT(""));
-         tags->AllowEditTitle(true);
-         tags->AllowEditTrackNumber(true);
-         if (!rval) {
-            return;
-         }
-      }
-   }
-
    bool ok;
 
    if (mLabel->GetValue()) {
@@ -492,12 +482,13 @@ bool ExportMultiple::DirOk()
 
 bool ExportMultiple::ExportMultipleByLabel(bool byName, wxString prefix)
 {
-   Track *tr;
+   AudacityProject *project = GetActiveProject();
+   bool tagsPrompt = project->GetShowId3Dialog();
    int numFiles = mNumLabels;
-   int l = 0;
-   int numLeft = 0;
-   int numRight = 0;
-   int numMono = 0;
+   int l = 0;        // counter for files done
+   ExportKitArray exportSettings; // dynamic array we will use to store the
+                                  // settings needed to do the exports with in
+   exportSettings.Alloc(numFiles);   // allocated some guessed space to use
 
    // Account for exporting before first label
    if (mFirst->GetValue()) {
@@ -505,102 +496,100 @@ bool ExportMultiple::ExportMultipleByLabel(bool byName, wxString prefix)
       numFiles++;
    }
 
-   // Figure out if we're exporting stereo or mono
-   for (tr = mIterator.First(mTracks); tr != NULL; tr = mIterator.Next()) {
+   // Figure out how many channels we should export
+   int channels = (mTracks->GetNumExportChannels(false));
+   
+   wxArrayString otherNames;  // keep track of file names we will use, so we
+                              // don't duplicate them
+   ExportKit setting;   // the current batch of settings
+   setting.destfile.SetPath(mDir->GetValue());
+   setting.destfile.SetExt(mPlugins[mFormatIndex]->GetExtension());
 
-      // Only want wave tracks
-      if (tr->GetKind() != Track::Wave) {
-         continue;
-      }
+   wxString name;    // used to hold file name whilst we mess with it
+   wxString title;   // un-messed-with title of file for tagging with
 
-      // Found a left channel
-      if (tr->GetChannel() == Track::LeftChannel) {
-         numLeft++;
-      }
-
-      // Found a right channel
-      else if (tr->GetChannel() == Track::RightChannel) {
-         numRight++;
-      }
-
-      // Found a mono channel, but it may be panned
-      else if (tr->GetChannel() == Track::MonoChannel) {
-         float pan = ((WaveTrack*)tr)->GetPan();
-
-         // Figure out what kind of channel it should be
-         if (pan == -1.0) {
-            numLeft++;
-         }
-         else if (pan == 1.0) {
-            numRight++;
-         }
-         else if (pan == 0) {
-            numMono++;
-         }
-         else {
-            numLeft++;
-            numRight++;
-         }
-      }
-   }
-
-   // We we have a stereo or mono mix?
-   bool stereo = false;
-   if (numRight > 0 || numLeft > 0) {
-      stereo = true;
-   }
-
-   wxArrayString otherNames;
-   wxString name;
-   double t0, t1;
-   int count = 0;
-   bool ok = true;
-
-   // Examine all labels
+   const LabelStruct *info = NULL;
+   /* Examine all labels a first time, sort out all data but don't do any
+    * exporting yet (so this run is quick but interactive) */
    while (l < mNumLabels) {
-      const LabelStruct *info = NULL;
-
+   
       // Get file name and starting time
       if (l < 0) {
-         name = mFirstFileName->GetValue();
-         t0 = 0.0;
+         // create wxFileName for output file 
+         name = (mFirstFileName->GetValue());
+         setting.t0 = 0.0;
       }
       else {
          info = mLabels->GetLabel(l);
-         name = info->title;
-         t0 = info->t;
+         name = (info->title);
+         setting.t0 = info->t;
       }
 
       // Figure out the ending time
       if (info && info->t < info->t1) {
-         t1 = info->t1;
+         setting.t1 = info->t1;
       }
       else if (l < mNumLabels-1) {
          const LabelStruct *info1 = mLabels->GetLabel(l+1);
-         t1 = info1->t;
+         setting.t1 = info1->t;
       }
       else {
-         t1 = mTracks->GetEndTime();
+         setting.t1 = mTracks->GetEndTime();
       }
 
+      // store title of label to use in tags
+      title = name;
+
       // Numbering files...
       if (!byName) {
-         name.Printf(wxT("%s-%d"), prefix.c_str(), count+1);
+         name.Printf(wxT("%s-%d"), prefix.c_str(), l+1);
       }
 
-      // Make sure the file name is unique within the directory
-      MakeNameUnique(otherNames, name);
-      
+      // store sanitised and user checjed name in object
+      setting.destfile.SetName(MakeFileName(name));
+
+      if (setting.destfile.GetName().IsEmpty())
+         {  // user cancelled dialogue, or deleted everything in feild.
+         // either way, cancel
+         return false;
+         }
+      wxASSERT(setting.destfile.IsOk());     // scream if file name is broke
+
+      // Make sure the (final) file name is unique within the set of exports
+      MakeNameUnique(otherNames, setting.destfile);
+
+      /* do the metadata for this file */
+      // copy project metadata to start with
+      setting.filetags = *(mProject->GetTags());
+      // over-ride with values
+      setting.filetags.SetTag(TAG_TITLE, title);
+      setting.filetags.SetTag(TAG_TRACK, l+1);
+      // let the user have a crack at editing it, exit if cancelled
+      if (!setting.filetags.ShowEditDialog(mProject,_("Edit metadata"), tagsPrompt))
+         return false;
+
+      /* add the settings to the array of settings to be used for export */
+      exportSettings.Add(setting);
+
+      l++;  // next label, count up one
+   }
+
+   bool ok = true;   // did it work?
+   int count = 0; // count the number of sucessful runs
+   ExportKit activeSetting;  // pointer to the settings in use for this export
+   /* Go round again and do the exporting (so this run is slow but
+    * non-interactive) */
+   for(count = 0; count < (numFiles); count++) {
+      /* get the settings to use for the export from the array */
+      activeSetting = exportSettings[count];
+
       // Export it
-      ok = DoExport(stereo, name, false, t0, t1, count+1);
+      ok = DoExport(channels, activeSetting.destfile, false, activeSetting.t0, activeSetting.t1, activeSetting.filetags);
       if (!ok) {
          break;
       }
-
-      count++;
-      l++;
    }
-   
+
    // Give 'em the result
    ::wxMessageBox(wxString::Format(ok
                                    ? _("Successfully exported %d file(s).")
@@ -615,16 +604,26 @@ bool ExportMultiple::ExportMultipleByLabel(bool byName, wxString prefix)
 bool ExportMultiple::ExportMultipleByTrack(bool byName,
                                            wxString prefix)
 {
+   AudacityProject *project = GetActiveProject();
+   bool tagsPrompt = project->GetShowId3Dialog();
    Track *tr, *tr2;
-   double t0, t1;
-   bool stereo;
+   int channels = 0;  // how many channels export?
+   int l = 0;     // track counter
    bool ok = true;
-   int i=0;
-   wxString name;
    wxArrayString otherNames;
-   wxArrayPtrVoid selected;
-   
-   // Remember which tracks were selected and set to unselected
+   wxArrayPtrVoid selected;   /**< Array of pointers to the tracks which were
+                                selected when we started */
+   ExportKitArray exportSettings; // dynamic array we will use to store the
+                                  // settings needed to do the exports with in
+   exportSettings.Alloc(mNumLabels);   // allocated some guessed space to use
+   ExportKit setting;   // the current batch of settings
+   setting.destfile.SetPath(mDir->GetValue());
+   setting.destfile.SetExt(mPlugins[mFormatIndex]->GetExtension());
+
+   wxString name;    // used to hold file name whilst we mess with it
+   wxString title;   // un-messed-with title of file for tagging with
+
+   /* Remember which tracks were selected, and set them to unselected */
    for (tr = mIterator.First(mTracks); tr != NULL; tr = mIterator.Next()) {
       if (tr->GetKind() != Track::Wave) {
          continue;
@@ -636,7 +635,7 @@ bool ExportMultiple::ExportMultipleByTrack(bool byName,
       }
    }
 
-   // Examine all tracks
+   /* Examine all tracks in turn, collecting export information */
    for (tr = mIterator.First(mTracks); tr != NULL; tr = mIterator.Next()) {
 
       // Only want wave tracks
@@ -645,11 +644,8 @@ bool ExportMultiple::ExportMultipleByTrack(bool byName,
       }
 
       // Get the times for the track
-      t0 = tr->GetStartTime();
-      t1 = tr->GetEndTime();
-
-      // Select the track
-      tr->SetSelected(true);
+      setting.t0 = tr->GetStartTime();
+      setting.t1 = tr->GetEndTime();
 
       // Check for a linked track
       tr2 = NULL;
@@ -657,38 +653,89 @@ bool ExportMultiple::ExportMultipleByTrack(bool byName,
          tr2 = mIterator.Next();
          if (tr2) {
 
-            // Select it also
-            tr2->SetSelected(true);
-
             // Make sure it gets included
-            if (tr2->GetStartTime() < t0) {
-               t0 = tr2->GetStartTime();
+            if (tr2->GetStartTime() < setting.t0) {
+               setting.t0 = tr2->GetStartTime();
             }
 
-            if (tr2->GetEndTime() > t1) {
-               t1 = tr2->GetEndTime();
+            if (tr2->GetEndTime() > setting.t1) {
+               setting.t1 = tr2->GetEndTime();
             }
          }
       }
 
-      // Stereo or mono
-      stereo = !(tr2 == NULL && 
-                 tr->GetChannel() == WaveTrack::MonoChannel &&
-                 ((WaveTrack *)tr)->GetPan() == 0.0);
+      // number of export channels?
+      if (tr2 == NULL && tr->GetChannel() == WaveTrack::MonoChannel &&
+                 ((WaveTrack *)tr)->GetPan() == 0.0)
+         channels = 1;
+      else
+         channels = 2;
 
-      // Get name base
+      // Get name and title
+      title = tr->GetName();
       if (byName) {
-         name = tr->GetName();
+         name = title;
       }
       else {
-         name.Printf(wxT("%s-%d"), prefix.c_str(), i+1);
+         name = (wxString::Format(wxT("%s-%d"), prefix.c_str(), l+1));
       }
 
-      // Make sure it's unique
-      MakeNameUnique(otherNames, name);
+      // store sanitised and user checked name in object
+      setting.destfile.SetName(MakeFileName(name));
 
-      // Go export it
-      ok = DoExport(stereo, name, true, t0, t1, i+1);
+      if (setting.destfile.GetName().IsEmpty())
+         {  // user cancelled dialogue, or deleted everything in feild.
+         // either way, cancel
+         return false;
+         }
+      wxASSERT(setting.destfile.IsOk());     // scream if file name is broke
+
+      // Make sure the (final) file name is unique within the set of exports
+      MakeNameUnique(otherNames, setting.destfile);
+
+      /* do the metadata for this file */
+      // copy project metadata to start with
+      setting.filetags = *(mProject->GetTags());
+      // over-ride with values
+      setting.filetags.SetTag(TAG_TITLE, title);
+      setting.filetags.SetTag(TAG_TRACK, l+1);
+      // let the user have a crack at editing it, exit if cancelled
+      if (!setting.filetags.ShowEditDialog(mProject,_("Edit metadata"), tagsPrompt))
+         return false;
+
+      /* add the settings to the array of settings to be used for export */
+      exportSettings.Add(setting);
+
+      l++;  // next label, count up one
+   }
+   // end of user-interactive data gathering loop, start of export processing
+   // loop
+   int count = 0; // count the number of sucessful runs
+   ExportKit activeSetting;  // pointer to the settings in use for this export
+   for (tr = mIterator.First(mTracks); tr != NULL; tr = mIterator.Next()) {
+
+      // Only want wave tracks
+      if (tr->GetKind() != Track::Wave) {
+         continue;
+      }
+
+      /* Select the track */
+      tr->SetSelected(true);
+
+      // Check for a linked track
+      tr2 = NULL;
+      if (tr->GetLinked()) {
+         tr2 = mIterator.Next();
+         if (tr2) {
+            // Select it also
+            tr2->SetSelected(true);
+         }
+      }
+
+      /* get the settings to use for the export from the array */
+      activeSetting = exportSettings[count];
+      // export data
+      ok = DoExport(channels, activeSetting.destfile, true, activeSetting.t0, activeSetting.t1, activeSetting.filetags);
 
       // Reset selection state
       tr->SetSelected(false);
@@ -700,8 +747,9 @@ bool ExportMultiple::ExportMultipleByTrack(bool byName,
       if (!ok) {
          break;
       }
+      // increment export counter
+      count++;
 
-      i++;
    }   
    
    // Restore the selection states
@@ -713,70 +761,86 @@ bool ExportMultiple::ExportMultipleByTrack(bool byName,
    ::wxMessageBox(wxString::Format(ok
                                    ? _("Successfully exported %d file(s).")
                                    : _("Something went wrong after exporting %d file(s)."),
-                                   i),
+                                   count),
                   _("Export Multiple"),
                   wxOK | wxCENTRE, this);
    
    return ok;
 }
 
-void ExportMultiple::MakeNameUnique(wxArrayString &otherNames, wxString &newName)
+void ExportMultiple::MakeNameUnique(wxArrayString &otherNames, wxFileName &newName)
 {
-   if (otherNames.Index(newName, false) >= 0) {
+   if (otherNames.Index(newName.GetFullName(), false) >= 0) {
       int i=2;
-      wxString orig = newName;
+      wxString orig = newName.GetName();
       do {
-         newName.Printf(wxT("%s-%d"), orig.c_str(), i);
+         newName.SetName(wxString::Format(wxT("%s-%d"), orig.c_str(), i));
          i++;
-      } while (otherNames.Index(newName, false) >= 0);
+      } while (otherNames.Index(newName.GetFullName(), false) >= 0);
    }
-   otherNames.Add(newName);
+   otherNames.Add(newName.GetFullName());
 }
 
-bool ExportMultiple::DoExport(bool stereo,
-                              wxString name,
+bool ExportMultiple::DoExport(int channels,
+                              wxFileName name,
                               bool selectedOnly,
                               double t0,
                               double t1,
-                              int trackNumber)
+                              Tags tags)
 {
-   wxFileName fn(mDir->GetValue(), name, mPlugins[mFormatIndex]->GetExtension());
+
 
    // Generate a unique name if we're not allowed to overwrite
    if (!mOverwrite->GetValue()) {
       int i = 2;
-      while (fn.FileExists()) {
-         fn.SetName(wxString::Format(wxT("%s-%d"), name.c_str(), i++));
+      wxString base(name.GetName());
+      while (name.FileExists()) {
+         name.SetName(wxString::Format(wxT("%s-%d"), base.c_str(), i++));
       }
    }
 
-   // If the format supports tags, then set the name and track number
-   Tags *tags = NULL;
-   wxString oldTitle;
-   wxString oldTrack;
-   if (mPlugins[mFormatIndex]->GetCanMetaData()) {
-      tags = mProject->GetTags();
-      oldTitle = tags->GetTag(TAG_TITLE);
-      oldTrack = tags->GetTag(TAG_TRACK);
-      tags->SetTag(TAG_TITLE, name);
-      tags->SetTag(TAG_TRACK, trackNumber);
-   }
-
    // Call the format export routine
    bool rc = mPlugins[mFormatIndex]->Export(mProject,
-                                            stereo ? 2 : 1,
-                                            fn.GetFullPath(),
+                                            channels,
+                                            name.GetFullPath(),
                                             selectedOnly,
                                             t0,
-                                            t1);
-   if (tags) {
-      tags->SetTag(TAG_TITLE, oldTitle);
-      tags->SetTag(TAG_TRACK, oldTrack);
-   }
-
+                                            t1,
+                                            NULL,
+                                            &tags);
    return rc;
 }
 
+wxString ExportMultiple::MakeFileName(wxString input)
+{
+   wxString newname; // name we are generating
+
+   // strip out anything that isn't allowed in file names on this platform
+   newname = Internat::SanitiseFilename(input, wxT("_"));
+
+   if(!newname.IsSameAs(input))
+   {  // need to get user to fix file name
+      // build the dialog
+      wxString msg;
+      msg.Printf(_("Label %s is not a legal file name. You cannot use any of:   %s\nUse..."), input.c_str(), wxFileName::GetForbiddenChars().c_str());
+      wxTextEntryDialog dlg( this, msg, _("Save As..."), newname );
+
+      // And tell the validator about excluded chars
+      dlg.SetTextValidator( wxFILTER_EXCLUDE_CHAR_LIST );
+      wxTextValidator *tv = dlg.GetTextValidator();
+      tv->SetExcludes(exclude);
+
+      // Show the dialog and bail if the user cancels
+      if( dlg.ShowModal() == wxID_CANCEL )
+      {
+          return wxEmptyString;
+      }
+      // Extract the name from the dialog
+      newname = dlg.GetValue();
+   }  // phew - end of file name sanitisation procedure
+   return newname;
+}
+
 // Indentation settings for Vim and Emacs and unique identifier for Arch, a
 // version control system. Please do not modify past this point.
 //
