@@ -703,6 +703,11 @@ void ControlToolBar::OnRecord(wxCommandEvent &evt)
       }
    }
 
+   if( evt.GetInt() == 1 ) // used when called by keyboard shortcut. Default (0) ignored.
+      mRecord->SetShift(true);
+   if( evt.GetInt() == 2 )
+      mRecord->SetShift(false);
+
    SetRecord(true);
 
    if (p) {
