@@ -63,11 +63,17 @@ of an LWSlider or ASlider.
 #include <wx/popupwin.h>
 #endif
 
+#if defined(__WXMAC__)
+#include <wx/sysopt.h>
+#endif
+
 #include "ASlider.h"
+#include "../Experimental.h"
 
 #include "../AColor.h"
 #include "../ImageManipulation.h"
 #include "../Project.h"
+#include "../ShuttleGui.h"
 
 #include "../Theme.h"
 #include "../AllThemeResources.h"
@@ -75,8 +81,6 @@ of an LWSlider or ASlider.
 //#include "../../images/SliderThumb.xpm"
 //#include "../../images/SliderThumbAlpha.xpm"
 
-#include <iostream>
-
 #if defined __WXMSW__
 const int sliderFontSize = 10;
 #else
@@ -89,6 +93,12 @@ const int sliderFontSize = 12;
 
 #if USE_POPUPWIN
 class TipPanel : public wxPopupWindow
+#elif defined(__WXMAC__)
+// On the Mac, we use a wxFrame as a wxPanel will appear to fall behind
+// whatever window it is hovering above if that window is refreshed.  A frame
+// does not have this problem.  One unfortunate side effect is that a shadow
+// now appears below the tip panel.
+class TipPanel : public wxFrame
 #else
 class TipPanel : public wxPanel
 #endif
@@ -102,6 +112,10 @@ class TipPanel : public wxPanel
 
    void OnPaint(wxPaintEvent & event);
 
+#if defined(__WXMAC__)
+   bool Show(bool show);
+#endif
+
    wxString label;
    wxString origLabel;
 
@@ -145,6 +159,17 @@ void TipPanel::SetPos(const wxPoint& pos)
 
 #else
 
+#if defined(__WXMAC__)
+BEGIN_EVENT_TABLE(TipPanel, wxFrame)
+   EVT_PAINT(TipPanel::OnPaint)
+END_EVENT_TABLE()
+
+TipPanel::TipPanel(wxWindow *parent, wxWindowID id,
+                   wxString label,
+                   const wxPoint &pos):
+   wxFrame(parent, id, wxEmptyString, wxDefaultPosition, wxDefaultSize,
+           wxNO_BORDER | wxFRAME_NO_TASKBAR)
+#else
 BEGIN_EVENT_TABLE(TipPanel, wxPanel)
    EVT_PAINT(TipPanel::OnPaint)
 END_EVENT_TABLE()
@@ -153,6 +178,8 @@ TipPanel::TipPanel(wxWindow *parent, wxWindowID id,
                    wxString label,
                    const wxPoint &pos):
    wxPanel(parent, id)
+#endif
+
 {
    this->label = label;
    this->origLabel = label;
@@ -175,6 +202,25 @@ void TipPanel::SetPos(const wxPoint& pos)
    Raise();
 }
 
+#if defined(__WXMAC__)
+bool TipPanel::Show(bool show)
+{
+   // Save original transition
+   int trans = wxSystemOptions::GetOptionInt( wxMAC_WINDOW_PLAIN_TRANSITION );
+
+   // Disable window animation
+   wxSystemOptions::SetOption( wxMAC_WINDOW_PLAIN_TRANSITION, 1 );
+
+   // Show/Hide the window
+   bool shown = wxFrame::Show(show);
+
+   // Disable window animation
+   wxSystemOptions::SetOption( wxMAC_WINDOW_PLAIN_TRANSITION, trans );
+
+   return shown;
+}
+#endif
+
 #endif
 
 void TipPanel::OnPaint(wxPaintEvent& event)
@@ -185,7 +231,11 @@ void TipPanel::OnPaint(wxPaintEvent& event)
    wxFont labelFont(sliderFontSize, wxSWISS, wxNORMAL, wxNORMAL);
    dc.SetFont(labelFont);
    GetClientSize(&width, &height);
+#if defined(__WXMAC__)
+   dc.SetPen(AColor::tooltipBrush.GetColour());
+#else
    dc.SetPen(*wxBLACK_PEN);
+#endif
    dc.SetBrush(AColor::tooltipBrush);
    dc.DrawRectangle(0, 0, width, height);
    dc.GetTextExtent(label, &textWidth, &textHeight);
@@ -227,7 +277,7 @@ SliderDialog::SliderDialog(wxWindow * parent, wxWindowID id,
    vs->Add(mSlider, 0, wxEXPAND | wxLEFT | wxRIGHT, 5 );
    
    //Create buttons 
-   vs->Add(CreateStdDialogButtonSizer(wxOK|wxCANCEL),0,wxEXPAND|wxALL,5);
+   vs->Add(CreateStdButtonSizer(this, eCancelButton|eOkButton), 0, wxEXPAND);
       
    //lay it out
    SetSizerAndFit(vs);
@@ -436,8 +486,11 @@ void LWSlider::SetPopWinPosition()
 {
    wxPoint pt(mWidth/2 + mLeft, mHeight + mTop + 1);
    pt = mParent->ClientToScreen(pt);
+
+#if !defined(__WXMAC__)
    if (GetToolTipParent())
       pt = GetToolTipParent()->ScreenToClient(pt);
+#endif
 
    if (mPopWin)
       ((TipPanel *)mPopWin)->SetPos(pt);
@@ -509,12 +562,64 @@ void LWSlider::Draw()
    // mask creation.  And garbage can be displayed if it isn't
    // cleared.
    dc->SelectObject(*mBitmap);
-   dc->SetBackground( wxBrush( wxColour( 255, 254, 255 ) ) );// DONT-THEME Mask colour.
+
+   wxColour TransparentColour = wxColour( 255, 254, 255 ); 
+   // DO-THEME Mask colour!!  JC-Aug-2007
+   // Needed with experimental theming!
+   //  ... because windows blends against this colour.
+#ifdef EXPERIMENTAL_THEMING
+   TransparentColour = theTheme.Colour( clrTrackInfo );
+#endif
+
+   dc->SetBackground( wxBrush( TransparentColour  ) );
    dc->Clear();
 
    AColor::Dark(dc, false);
    dc->DrawLine(mLeftX, mCenterY+1, mRightX+2, mCenterY+1);
 
+   // Draw +/- or L/R first.  We need to draw these before the tick marks.
+   if (mStyle == PAN_SLIDER)
+   {
+      // sliderFontSize is for the tooltip.
+      // we need something smaller here...
+      wxFont labelFont(sliderFontSize-3, wxSWISS, wxNORMAL, wxNORMAL);
+      dc->SetFont(labelFont);
+
+#ifdef EXPERIMENTAL_THEMING
+      dc->SetTextForeground( theTheme.Colour( clrTrackPanelText ));
+      // TransparentColour should be same as clrTrackInfo.
+      dc->SetTextBackground( theTheme.Colour( clrTrackInfo ) );
+      dc->SetBackground( theTheme.Colour( clrTrackInfo ) );
+      // HAVE to use solid and not transparent here,
+      // otherwise windows will do it's clever font optimisation trick, 
+      // but against a default colour of white, which is not OK on a dark
+      // background.
+      dc->SetBackgroundMode( wxSOLID );
+#else
+      dc->SetTextForeground( wxColour( 0,0,0) );
+      dc->SetTextBackground( wxColour( 255,255,255));
+#endif
+
+      /* i18n-hint: One-letter abbreviation for Left, in the Pan slider */
+      dc->DrawText(_("L"), mLeftX, 0);
+
+      /* i18n-hint: One-letter abbreviation for Right, in the Pan slider */
+      dc->DrawText(_("R"), mRightX-6,0);
+   } else
+   {
+      // draw the '-' and the '+'
+#ifdef EXPERIMENTAL_THEMING
+      wxPen pen( theTheme.Colour( clrTrackPanelText ));
+      dc->SetPen( pen );
+#else
+      dc->SetPen(*wxBLACK_PEN);
+#endif
+      dc->DrawLine(mLeftX, mCenterY-10, mLeftX+5, mCenterY-10);
+      dc->DrawLine(mRightX-5, mCenterY-10, mRightX, mCenterY-10);
+      dc->DrawLine(mRightX-3, mCenterY-12, mRightX-3, mCenterY-7);
+   }
+
+
    int divs = 10;
    double upp = divs / (double)(mWidthX-1);
    double d = 0;
@@ -531,30 +636,12 @@ void LWSlider::Draw()
       d += upp;
    }
 
-   if (mStyle == PAN_SLIDER)
-   {
-      wxFont labelFont(sliderFontSize, wxSWISS, wxNORMAL, wxNORMAL);
-      dc->SetFont(labelFont);
-
-      /* i18n-hint: One-letter abbreviation for Left, in the Pan slider */
-      dc->DrawText(_("L"), mLeftX, 1);
-
-      /* i18n-hint: One-letter abbreviation for Right, in the Pan slider */
-      dc->DrawText(_("R"), mRightX-7, 1);
-   } else
-   {
-      // draw the '-' and the '+'
-      dc->SetPen(*wxBLACK_PEN);
-      dc->DrawLine(mLeftX, mCenterY-10, mLeftX+5, mCenterY-10);
-      dc->DrawLine(mRightX-7, mCenterY-10, mRightX-2, mCenterY-10);
-      dc->DrawLine(mRightX-5, mCenterY-12, mRightX-5, mCenterY-7);
-   }
    
    // Must preceed creating the mask as that will attempt to
    // select the bitmap into another DC.
    delete dc;
 
-   mBitmap->SetMask( new wxMask( *mBitmap, wxColour( 255, 254, 255 ) ) );
+   mBitmap->SetMask( new wxMask( *mBitmap, TransparentColour ) );
 }
 
 
@@ -702,7 +789,8 @@ void LWSlider::OnMouseEvent(wxMouseEvent & event)
    else if( event.ButtonUp() )
    {
       mIsDragging = false;
-      mParent->ReleaseMouse();
+      if (mParent->HasCapture())
+         mParent->ReleaseMouse();
       mPopWin->Hide();
       ((TipPanel *)mPopWin)->SetPos(wxPoint(-1000, -1000));
    }
@@ -1050,6 +1138,11 @@ void ASlider::OnPaint(wxPaintEvent &event)
 {
    wxPaintDC dc(this);
 
+#ifdef EXPERIMENTAL_THEMING
+   wxColour Col(GetParent()->GetBackgroundColour());
+   this->SetBackgroundColour( Col );
+#endif
+
    mLWSlider->OnPaint(dc, false);
 
    if( mSliderIsFocused )
