@@ -168,6 +168,14 @@ different formats.
 #include <wx/tooltip.h>
 #include <wx/toplevel.h>
 
+#ifdef _DEBUG
+    #ifdef _MSC_VER
+        #undef THIS_FILE
+        static char*THIS_FILE= __FILE__;
+        #define new new(_NORMAL_BLOCK, THIS_FILE, __LINE__)
+    #endif
+#endif
+
 #define ID_MENU 9800
 
 // Custom events
@@ -188,49 +196,7 @@ END_EVENT_TABLE()
 
 IMPLEMENT_CLASS(TimeTextCtrl, wxControl)
 
-struct BuiltinFormatString {
-   const wxChar *name;
-   const wxChar *formatStr;
-};
 
-BuiltinFormatString BuiltinFormatStrings[] =
-   {{_("seconds"),
-/* i18n-hint: These strings control the formatting of time units in the 
- * display. They are fairly complex, see the comments in
- * widgets/TimeTextCtrl.cpp for how they work */
-     _("01000,01000 seconds")},
-    {_("hh:mm:ss"),
-     _("0100 h 060 m 060 s")},
-    {_("dd:hh:mm:ss"),
-     _("0100 days 024 h 060 m 060 s")},
-    {_("hh:mm:ss + milliseconds"),
-     _("0100 h 060 m 060.01000 s")},
-    {_("hh:mm:ss + samples"),
-     _("0100 h 060 m 060 s+.# samples")},
-    {_("samples"),
-     _("01000,01000,01000 samples|#")},
-    {_("hh:mm:ss + film frames (24 fps)"),
-     _("0100 h 060 m 060 s+.24 frames")},
-    {_("film frames (24 fps)"),
-     _("01000,01000 frames|24")},
-    {_("hh:mm:ss + NTSC drop frames"),
-     _("0100 h 060 m 060 s+.30 frames|N")},
-    {_("hh:mm:ss + NTSC non-drop frames"),
-     _("0100 h 060 m 060 s+.030 frames| .999000999")},
-    {_("NTSC frames"),
-     _("01000,01000 frames|29.97002997")},
-    {_("hh:mm:ss + PAL frames (25 fps)"),
-     _("0100 h 060 m 060 s+.25 frames")},
-    {_("PAL frames (25 fps)"),
-     _("01000,01000 frames|25")},
-    {_("hh:mm:ss + CDDA frames (75 fps)"),
-     _("0100 h 060 m 060 s+.75 frames")},
-    {_("CDDA frames (75 fps)"),
-     _("01000,01000 frames|75")}};
-
-const int kNumBuiltinFormatStrings =
-   sizeof(BuiltinFormatStrings) /
-   sizeof(BuiltinFormatStrings[0]);
 
 class TimeField {
 public:
@@ -256,7 +222,6 @@ public:
       if (zeropad && range>1)
          formatStr.Printf(wxT("%%0%dd"), digits); // ex. "%03d" if digits is 3
       else {
-         //formatStr = wxT("%d");/////////////old code
          formatStr.Printf(wxT("%%0%dd"), digits);
       }
    }
@@ -294,6 +259,126 @@ TimeTextCtrl::TimeTextCtrl(wxWindow *parent,
    mFocusedDigit(0),
    mLastField(-1)
 {
+   /* i18n-hint: Name of time display format that shows time in seconds */
+   BuiltinFormatStrings[0].name = _("seconds");
+   /* i18n-hint: Format string for displaying time in seconds. Change the comma
+    * in the middle to the 1000s separator for your locale, and the 'seconds'
+    * on the end to the word for seconds. Don't change the numbers. */
+   BuiltinFormatStrings[0].formatStr = _("01000,01000 seconds");
+   /* i18n-hint: Name of time display format that shows time in hours, minutes
+    * and seconds */
+   BuiltinFormatStrings[1].name = _("hh:mm:ss");
+   /* i18n-hint: Format string for displaying time in hours, minutes and
+    * seconds. Change the 'h' to the abbreviation for hours, 'm' to the
+    * abbreviation for minutes and 's' to the abbreviation for seconds. Don't
+    * change the numbers unless there aren't 60 seconds in a minute in your
+    * locale */
+   BuiltinFormatStrings[1].formatStr = _("0100 h 060 m 060 s");
+   /* i18n-hint: Name of time display format that shows time in days, hours,
+    * minutes and seconds */
+   BuiltinFormatStrings[2].name = _("dd:hh:mm:ss");
+   /* i18n-hint: Format string for displaying time in days, hours, minutes and
+    * seconds. Change the 'days' to the word for days, 'h' to the abbreviation
+    * for hours, 'm' to the abbreviation for minutes and 's' to the
+    * abbreviation for seconds. Don't change the numbers unless there aren't
+    * 24 hours in a day in your locale */
+   BuiltinFormatStrings[2].formatStr = _("0100 days 024 h 060 m 060 s");
+   /* i18n-hint: Name of time display format that shows time in hours,
+    * minutes, seconds and milliseconds (1/1000 second) */
+   BuiltinFormatStrings[3].name = _("hh:mm:ss + milliseconds");
+   /* i18n-hint: Format string for displaying time in hours, minutes, seconds
+    * and milliseconds. Change the 'h' to the abbreviation for hours, 'm' to the
+    * abbreviation for minutes and 's' to the abbreviation for seconds (the
+    * milliseconds are shown as decimal seconds) . Don't change the numbers
+    * unless there aren't 60 minutes in an hour in your locale */
+   BuiltinFormatStrings[3].formatStr =_("0100 h 060 m 060.01000 s");
+   /* i18n-hint: Name of time display format that shows time in hours,
+    * minutes, seconds and samples (at the current project sample rate) */
+   BuiltinFormatStrings[4].name = _("hh:mm:ss + samples");
+   /* i18n-hint: Format string for displaying time in hours, minutes, seconds
+    * and samples. Change the 'h' to the abbreviation for hours, 'm' to the
+    * abbreviation for minutes, 's' to the abbreviation for seconds and
+    * translate samples . Don't change the numbers
+    * unless there aren't 60 seconds in a minute in your locale */
+   BuiltinFormatStrings[4].formatStr = _("0100 h 060 m 060 s+.# samples");
+   /* i18n-hint: Name of time display format that shows time in samples (at the
+    * current project sample rate) */
+   BuiltinFormatStrings[5].name = _("samples");
+   /* i18n-hint: Format string for displaying time in samples (lots of samples).
+    * Change the ',' to the 1000s separator for your locale, and translate
+    * samples. If 1000s aren't a base multiple for your number system, then you
+    * can change the numbers to an appropriate one, and put a 0 on the front */
+   BuiltinFormatStrings[5].formatStr = _("01000,01000,01000 samples|#");
+   /* i18n-hint: Name of time display format that shows time in hours, minutes,
+    * seconds and frames at 24 frames per second (commonly used for films) */
+   BuiltinFormatStrings[6].name = _("hh:mm:ss + film frames (24 fps)");
+   /* i18n-hint: Format string for displaying time in hours, minutes, seconds
+    * and frames at 24 frames per second. Change the 'h' to the abbreviation
+    * for hours, 'm' to the abbreviation for minutes, 's' to the abbreviation
+    * for seconds and translate 'frames' . Don't change the numbers
+    * unless there aren't 60 seconds in a minute in your locale */
+   BuiltinFormatStrings[6].formatStr = _("0100 h 060 m 060 s+.24 frames");
+   /* i18n-hint: Name of time display format that shows time in frames (lots of
+    * frames) at 24 frames per second (commonly used for films) */
+   BuiltinFormatStrings[7].name = _("film frames (24 fps)");
+   /* i18n-hint: Format string for displaying time in frames at 24 frames per
+    * second. Translate 'frames' and leave the rest alone */
+   BuiltinFormatStrings[7].formatStr = _("01000,01000 frames|24");
+   /* i18n-hint: Name of time display format that shows time in hours, minutes,
+    * seconds and frames at NTSC TV drop-frame rate (used for American /
+    * Japananese TV, and very odd) */
+   BuiltinFormatStrings[8].name = _("hh:mm:ss + NTSC drop frames");
+   /* i18n-hint: Format string for displaying time in hours, minutes, seconds
+    * and frames with NTSC drop frames. Change the 'h' to the abbreviation
+    * for hours, 'm' to the abbreviation for minutes, 's' to the abbreviation
+    * for seconds and translate 'frames'. Leave the |N alone, it's important! */
+   BuiltinFormatStrings[8].formatStr = _("0100 h 060 m 060 s+.30 frames|N");
+   /* i18n-hint: Name of time display format that shows time in hours, minutes,
+    * seconds and frames at NTSC TV non-drop-frame rate (used for American /
+    * Japananese TV, and doesn't quite match wall time */
+   BuiltinFormatStrings[9].name = _("hh:mm:ss + NTSC non-drop frames");
+   /* i18n-hint: Format string for displaying time in hours, minutes, seconds
+    * and frames with NTSC drop frames. Change the 'h' to the abbreviation
+    * for hours, 'm' to the abbreviation for minutes, 's' to the abbreviation
+    * for seconds and translate 'frames'. Leave the | .999000999 alone, 
+    * the whole things really is slightly off-speed! */
+   BuiltinFormatStrings[9].formatStr = _("0100 h 060 m 060 s+.030 frames| .999000999");
+   /* i18n-hint: Name of time display format that shows time in frames at NTSC
+    * TV frame rate (used for American / Japananese TV */
+   BuiltinFormatStrings[10].name = _("NTSC frames");
+   /* i18n-hint: Format string for displaying time in frames with NTSC frames.
+    * Translate 'frames' and leave the rest alone. That really is the frame
+    * rate! */
+   BuiltinFormatStrings[10].formatStr = _("01000,01000 frames|29.97002997");
+   /* i18n-hint: Name of time display format that shows time in hours, minutes,
+    * seconds and frames at PAL TV frame rate (used for European TV) */
+   BuiltinFormatStrings[11].name = _("hh:mm:ss + PAL frames (25 fps)");
+   /* i18n-hint: Format string for displaying time in hours, minutes, seconds
+    * and frames with PAL TV frames. Change the 'h' to the abbreviation
+    * for hours, 'm' to the abbreviation for minutes, 's' to the abbreviation
+    * for seconds and translate 'frames'. Nice simple time code! */
+   BuiltinFormatStrings[11].formatStr = _("0100 h 060 m 060 s+.25 frames");
+   /* i18n-hint: Name of time display format that shows time in frames at PAL
+    * TV frame rate (used for European TV */
+   BuiltinFormatStrings[12].name = _("PAL frames (25 fps)");
+   /* i18n-hint: Format string for displaying time in frames with NTSC frames.
+    * Translate 'frames' and leave the rest alone. */
+   BuiltinFormatStrings[12].formatStr = _("01000,01000 frames|25");
+   /* i18n-hint: Name of time display format that shows time in hours, minutes,
+    * seconds and frames at CD Audio frame rate (75 frames per second) */
+   BuiltinFormatStrings[13].name = _("hh:mm:ss + CDDA frames (75 fps)");
+   /* i18n-hint: Format string for displaying time in hours, minutes, seconds
+    * and frames with CD Audio frames. Change the 'h' to the abbreviation
+    * for hours, 'm' to the abbreviation for minutes, 's' to the abbreviation
+    * for seconds and translate 'frames'. */
+   BuiltinFormatStrings[13].formatStr = _("0100 h 060 m 060 s+.75 frames");
+   /* i18n-hint: Name of time display format that shows time in frames at CD
+    * Audio frame rate (75 frames per second) */
+   BuiltinFormatStrings[14].name = _("CDDA frames (75 fps)");
+   /* i18n-hint: Format string for displaying time in frames with CD Audio
+    * frames. Translate 'frames' and leave the rest alone */
+   BuiltinFormatStrings[14].formatStr = _("01000,01000 frames|75");
+
    mDigitBoxW = 10;
    mDigitBoxH = 16;
 
@@ -384,22 +469,42 @@ const double TimeTextCtrl::GetTimeValue()
    return mTimeValue;
 }
 
-int TimeTextCtrl::GetNumBuiltins()
+wxString TimeTextCtrl::GetFormatString()
 {
-   return kNumBuiltinFormatStrings;
+   return mFormatString;
 }
 
-wxString TimeTextCtrl::GetBuiltinName(int index)
+int TimeTextCtrl::GetFormatIndex()
 {
-   if (index >= 0 && index < kNumBuiltinFormatStrings)
+   int ndx = 1;
+   int i;
+
+   for (i = 0; i < TimeTextCtrl::GetNumBuiltins(); i++) {
+      if (mFormatString == TimeTextCtrl::GetBuiltinFormat(i)) {
+         ndx = i;
+         break;
+      }
+   }
+
+   return ndx;
+}
+   
+int TimeTextCtrl::GetNumBuiltins()
+{
+   return (sizeof(BuiltinFormatStrings) / sizeof(BuiltinFormatStrings[0]));
+}
+
+wxString TimeTextCtrl::GetBuiltinName(const int index)
+{
+   if (index >= 0 && index < GetNumBuiltins())
       return BuiltinFormatStrings[index].name;
    else
       return wxT("");
 }
 
-wxString TimeTextCtrl::GetBuiltinFormat(int index)
+wxString TimeTextCtrl::GetBuiltinFormat(const int index)
 {
-   if (index >= 0 && index < kNumBuiltinFormatStrings)
+   if (index >= 0 && index < GetNumBuiltins())
       return BuiltinFormatStrings[index].formatStr;
    else
       return wxT("");
@@ -731,7 +836,7 @@ void TimeTextCtrl::Fit()
    sz.x = mButtonWidth + mWidth + (sz.x - csz.x);
    sz.y = mHeight + (sz.y - csz.y);
 
-   SetBestFittingSize(sz);
+   SetBestFittingSize(sz); // update when we drop wx 2.6 support
 }
 
 void TimeTextCtrl::OnErase(wxEraseEvent & event)
